#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  instrument_handler.py
#   Purpose:   handling instrument correction in obspyDMT
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
from datetime import datetime
import fnmatch
import multiprocessing
from obspy.core import read
from obspy import read_inventory
try:
    from obspy.io.xseed import Parser
except Exception, e:
    from obspy.xseed import Parser
import os
import sys

from event_handler import quake_info
from utility_codes import read_station_event, check_par_jobs

# ##################### FDSN_ARC_IC #####################################


def FDSN_ARC_IC(input_dics, clients):
    """
    Call "inst_correct" function based on the request.
    Group the stations that have been retrieved from specific client
    Grouping is necessary in applying the instrument correction correctly
    (different clients are treated differently)
    :param input_dics:
    :param clients:
    :return:
    """
    print '\n*****************************'
    print '%s -- Instrument Correction' % clients
    print '*****************************'
    # Following two if-conditions create address to which
    # instrument correction should be applied
    # Please note that these two conditions can not happen at the same time
    address = None
    if clients == 'arc':
        clients_name = 'arc'
    else:
        clients_name = 'fdsn'
    if input_dics[clients_name + '_ic_auto'] == 'Y':
        period = '{0:s}_{1:s}'.\
            format(input_dics['min_date'].split('T')[0],
                   input_dics['max_date'].split('T')[0])
        address = os.path.join(input_dics['datapath'], period)

    if input_dics['fdsn_update'] != 'N':
        address = os.path.join(input_dics['datapath'])

    if input_dics['arc_update'] != 'N':
        address = os.path.join(input_dics['datapath'])

    if input_dics[clients_name + '_ic'] != 'N':
        address = input_dics[clients_name + '_ic']

    events, address_events = quake_info(address, 'info')

    for i in range(len(events)):
        sta_ev = read_station_event(address_events[i])
        ls_saved_stas_tmp = []
        for s_ev in sta_ev[0]:
            if clients.lower() == 'all_fdsn':
                if not s_ev[13].lower() == 'arc':
                    station_id = '%s.%s.%s.%s' % (s_ev[0], s_ev[1],
                                                  s_ev[2], s_ev[3])
                    ls_saved_stas_tmp.append(
                        os.path.join(address_events[i], 'BH_RAW', station_id))
            elif clients.lower() == s_ev[13].lower():
                station_id = '%s.%s.%s.%s' % (s_ev[0], s_ev[1],
                                              s_ev[2], s_ev[3])
                ls_saved_stas_tmp.append(
                    os.path.join(address_events[i], 'BH_RAW', station_id))

        if input_dics['net'].startswith('_'):
            net_pat = '*'
        else:
            net_pat = input_dics['net']

        ls_saved_stas = []
        for net_iter in net_pat.split(','):
            for sta_iter in input_dics['sta'].split(','):
                for loc_iter in input_dics['loc'].split(','):
                    for cha_iter in input_dics['cha'].split(','):
                        pattern_sta = '%s.%s.%s.%s' % (net_iter, sta_iter,
                                                       loc_iter, cha_iter)
                        for saved_sta in ls_saved_stas_tmp:
                            if fnmatch.fnmatch(os.path.basename(saved_sta),
                                               pattern_sta):
                                ls_saved_stas.append(saved_sta)

        if len(ls_saved_stas) != 0:
            print '\nevent: %s/%s -- %s\n' % (i+1, len(events), clients)
            inst_correct(input_dics, ls_saved_stas, address_events[i], clients)
        else:
            print "There is no station in the directory to correct!"
            print "Address: %s" % address_events[i]
    # pass address for using in create_tar_file
    return address

# ##################### inst_correct ###############################


def inst_correct(input_dics, ls_saved_stas, address, clients):
    """
    Apply Instrument Coorection on all available stations in the folder

    Instrument Correction has three main steps:
        1) RTR: remove the trend
        2) tapering
        3) pre-filtering and deconvolution of Resp file from Raw counts

    Remove the instrument type by deconvolution using spectral division.
    :param input_dics:
    :param ls_saved_stas:
    :param address:
    :param clients:
    :return:
    """
    t_inst_1 = datetime.now()

    if not input_dics['corr_unit'] in ['DIS', 'VEL', 'ACC']:
        sys.exit('ERROR: %s can not be recognized.\n'
                 'Available options: DIS, VEL, ACC' % input_dics['corr_unit'])
    if input_dics['corr_unit'] == 'DIS':
        BH_file = 'BH'
    else:
        BH_file = 'BH_' + input_dics['corr_unit']

    try:
        os.makedirs(os.path.join(address, BH_file))
    except Exception as e:
        print "\nWARNING: can not create %s" % os.path.join(address, BH_file)
        print "%s\n" % e
        pass

    ic_serial_parallel(input_dics, ls_saved_stas, clients, address, BH_file)
    t_inst_2 = datetime.now()

    if input_dics['ic_parallel'] == 'Y':
        ic_reporter(input_dics, address, clients, ls_saved_stas, t_inst_1,
                    t_inst_2)

    print '\nTime for instrument correction of %s channels: %s' \
          % (len(ls_saved_stas), t_inst_2-t_inst_1)

# ##################### ic_serial_parallel ###################################


def ic_serial_parallel(input_dics, ls_saved_stas, clients, address, BH_file):
    """
    instrument correction in serial/parallel
    :param input_dics:
    :param ls_saved_stas:
    :param clients:
    :param address:
    :param BH_file:
    :return:
    """

    if input_dics['ic_parallel'] == 'Y':
        print '\nParallel instrument correction with %s processes.\n' \
              % input_dics['ic_np']

        start = 0
        end = len(ls_saved_stas)
        step = (end - start) / input_dics['ic_np'] + 1

        jobs = []
        for index in xrange(input_dics['ic_np']):
            starti = start+index*step
            endi = min(start+(index+1)*step, end)
            print starti, endi
            p = multiprocessing.Process(target=IC_core_iterate,
                                        args=(input_dics, ls_saved_stas,
                                              clients, address, BH_file,
                                              starti, endi))
            jobs.append(p)
        for i in range(len(jobs)):
            jobs[i].start()
        check_par_jobs(jobs)

    else:
        for i in range(len(ls_saved_stas)):
            IC_core(input_dics=input_dics, ls_saved_sta=ls_saved_stas[i],
                    clients=clients, address=address, BH_file=BH_file,
                    inform='%s -- %s/%s' % (clients, i+1, len(ls_saved_stas)))

# ##################### IC_core_iterate ######################################


def IC_core_iterate(input_dics, ls_saved_stas, clients, address, BH_file,
                    starti, endi):
    """
    Simple iterator over IC_core Designed to be used for
    parallel instrument correction
    :param input_dics:
    :param ls_saved_stas:
    :param clients:
    :param address:
    :param BH_file:
    :param starti:
    :param endi:
    :return:
    """
    for i in range(starti, endi):
        IC_core(input_dics=input_dics, ls_saved_sta=ls_saved_stas[i],
                clients=clients, address=address, BH_file=BH_file,
                inform='%s -- %s/%s' % (clients, i+1, len(ls_saved_stas)))

# ##################### IC_core #########################################


def IC_core(input_dics, ls_saved_sta, clients, address, BH_file, inform):
    """
    Function that prepare the waveforms for instrument correction and
    divert the program to the right instrument correction function!
    :param input_dics:
    :param ls_saved_sta:
    :param clients:
    :param address:
    :param BH_file:
    :param inform:
    :return:
    """
    try:
        if input_dics['ic_obspy_full'] == 'Y':
            st = read(ls_saved_sta)
            if len(st) > 1:
                print "WARNING: %s\nprobably has some gaps" % ls_saved_sta
                print "\nIt will be merged (fill_value=0) before " \
                      "applying the instrument correction"
                st.merge(method=1, fill_value=0, interpolation_samples=0)
            tr = st[0]
            if clients.lower() != 'arc':
                stxml_file = \
                    os.path.join(address, 'Resp',
                                 'STXML.%s' % os.path.basename(ls_saved_sta))
                obspy_fullresp_STXML(input_dics=input_dics, trace=tr,
                                     stxml_file=stxml_file,
                                     Address=os.path.join(address, BH_file),
                                     unit=input_dics['corr_unit'],
                                     BP_filter=input_dics['pre_filt'],
                                     inform=inform)
            else:
                resp_file = \
                    os.path.join(address, 'Resp',
                                 'DATALESS.%s'
                                 % os.path.basename(ls_saved_sta))
                obspy_fullresp_RESP(input_dics=input_dics, trace=tr,
                                    resp_file=resp_file,
                                    Address=os.path.join(address, BH_file),
                                    unit=input_dics['corr_unit'],
                                    BP_filter=input_dics['pre_filt'],
                                    inform=inform)
        else:
            sys.exit('ERROR: only ic_obspy_full is implemented for '
                     'instrument correction')
    except Exception as e:
        print e

# ##################### obspy_fullresp_STXML #################################


def obspy_fullresp_STXML(input_dics, trace, stxml_file, Address, unit='DIS',
                         BP_filter=(0.008, 0.012, 3.0, 4.0), inform='N/N'):
    """
    Instrument correction using station_XML --->
    equivalent to full response file steps: detrend, demean, taper, filter,
    deconvolution
    :param input_dics:
    :param trace:
    :param stxml_file:
    :param Address:
    :param unit:
    :param BP_filter:
    :param inform:
    :return:
    """
    try:
        trace.detrend('linear')
        # To keep it consistant with obspy.remove_response method!
        if unit.lower() == 'dis':
            unit = 'DISP'
            unit_write = 'dis'
        else:
            unit_write = unit.lower()

        inv = read_inventory(stxml_file, format="stationxml")
        trace.attach_response(inv)
        trace.remove_response(output=unit,
                              water_level=input_dics['water_level'],
                              pre_filt=eval(BP_filter), zero_mean=True,
                              taper=True, taper_fraction=0.05)
        # Remove the following line to keep the units
        # as it is in the stationXML
        # trace.data *= 1.e9

        trace_identity = '%s.%s.%s.%s' % (trace.stats['network'],
                                          trace.stats['station'],
                                          trace.stats['location'],
                                          trace.stats['channel'])
        if input_dics['mseed'] == 'N':
            trace.write(os.path.join(
                Address, '%s.%s' % (unit_write, trace_identity)), format='SAC')
        else:
            trace.write(
                os.path.join(Address, '%s.%s' % (unit_write, trace_identity)),
                format='MSEED')

        if unit.lower() == 'disp':
            unit_print = 'displacement'
        elif unit.lower() == 'vel':
            unit_print = 'velocity'
        elif unit.lower() == 'acc':
            unit_print = 'acceleration'
        else:
            unit_print = 'UNKNOWN'
        print '%s -- instrument correction to %s for: %s' \
              % (inform, unit_print, trace_identity)

    except Exception as e:
        print '%s -- %s' % (inform, e)

# ##################### obspy_fullresp_RESP ##################################


def obspy_fullresp_RESP(input_dics, trace, resp_file, Address, unit='DIS',
                        BP_filter=(0.008, 0.012, 3.0, 4.0), inform='N/N'):
    """
    Instrument correction using dataless seed --->
    equivalent to full response file steps: detrend, demean, taper, filter,
    deconvolution
    :param input_dics:
    :param trace:
    :param resp_file:
    :param Address:
    :param unit:
    :param BP_filter:
    :param inform:
    :return:
    """
    dataless_parser = Parser(resp_file)
    seedresp = {'filename': dataless_parser, 'units': unit}

    try:
        trace.detrend('linear')
        trace.simulate(seedresp=seedresp, paz_remove=None, paz_simulate=None,
                       remove_sensitivity=True, simulate_sensitivity=False,
                       water_level=input_dics['water_level'],
                       zero_mean=True, taper=True,
                       taper_fraction=0.05, pre_filt=eval(BP_filter),
                       pitsasim=False, sacsim=True)
        # Remove the following line since we want to keep
        # the units as it is in the stationXML
        # trace.data *= 1.e9
        trace_identity = '%s.%s.%s.%s' % (trace.stats['network'],
                                          trace.stats['station'],
                                          trace.stats['location'],
                                          trace.stats['channel'])
        if input_dics['mseed'] == 'N':
            trace.write(os.path.join(Address, '%s.%s' % (unit.lower(),
                                                         trace_identity)),
                        format='SAC')
        else:
            trace.write(os.path.join(Address, '%s.%s' % (unit.lower(),
                                                         trace_identity)),
                        format='MSEED')

        if unit.lower() == 'dis':
            unit_print = 'displacement'
        elif unit.lower() == 'vel':
            unit_print = 'velocity'
        elif unit.lower() == 'acc':
            unit_print = 'acceleration'
        else:
            unit_print = 'UNKNOWN'
        print '%s -- instrument correction to %s for: %s' \
              % (inform, unit_print, trace_identity)

    except Exception as e:
        print '%s -- %s' % (inform, e)

# ##################### ic_reporter ##################################


def ic_reporter(input_dics, address, clients, ls_saved_stas, t_inst_1,
                t_inst_2):
    """
    writing log files for instrument correction
    :param input_dics:
    :param address:
    :param clients:
    :param ls_saved_stas:
    :param t_inst_1:
    :param t_inst_2:
    :return:
    """
    report_parallel_open = open(os.path.join(address, 'info',
                                             'report_parallel'), 'a')
    report_parallel_open.writelines('---------------%s---------------\n'
                                    % clients.upper())
    report_parallel_open.writelines('Instrument Correction\n')
    report_parallel_open.writelines('Number of Nodes: %s\n'
                                    % input_dics['ic_np'])
    report_parallel_open.writelines('Number of Stas : %s\n'
                                    % len(ls_saved_stas))
    report_parallel_open.writelines('Total Time     : %s\n'
                                    % (t_inst_2 - t_inst_1))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  obspyDMT.py
#   Purpose:   obspyDMT main program 
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------
import sys
import time

from utils.arclink_handler import ARC_network
from utils.event_handler import get_Events, create_tar_file
from utils.fdsn_handler import FDSN_network
from utils.input_handler import command_parse, read_input_command
from utils.instrument_handler import FDSN_ARC_IC
from utils.merge_handler import FDSN_ARC_merge
from utils.plotting_tools import plot_xml_response, seismicity, plot_tools
from utils.update_handler import FDSN_update, ARC_update
from utils.utility_codes import header_printer, send_email, goodbye_printer

# ##################### obspyDMT ##################################


def obspyDMT(**kwargs):
    """
    obspyDMT: is the function dedicated to the main part of the code.
    It organizes all the sub-main functions to run the program
    :param kwargs:
    :return:
    """
    # printing the header
    header_printer()
    # initializing variables:
    events = None
    create_tar_file_address = None
    # ------------------Parsing command-line options--------------------
    (options, args, parser) = command_parse()
    # ------------------Read INPUT file (Parameters)--------------------
    input_dics = read_input_command(parser, **kwargs)
    # ------------------plot stationxml files--------------------
    if input_dics['plotxml_dir']:
        plot_xml_response(input_dics)
    # ------------------Getting List of Events/Continuous requests------
    if input_dics['get_events'] == 'Y':
        events = get_Events(input_dics, request='event-based')
        if events == 0:
            return input_dics
    if input_dics['get_continuous'] == 'Y':
        events = get_Events(input_dics, request='continuous')
    # ------------------Seismicity--------------------------------------
    if input_dics['seismicity'] == 'Y':
        seismicity(input_dics, events)
    # ------------------FDSN--------------------------------------------
    if input_dics['FDSN'] == 'Y':
        FDSN_network(input_dics, events)
    # ------------------Arclink-----------------------------------------
    if input_dics['ArcLink'] == 'Y':
        ARC_network(input_dics, events)
    # ------------------Update-----------------------------------
    if input_dics['fdsn_update'] != 'N':
        FDSN_update(input_dics, address=input_dics['fdsn_update'])
    if input_dics['arc_update'] != 'N':
        ARC_update(input_dics, address=input_dics['arc_update'])
    # ------------------instrument---------------------------------
    if input_dics['fdsn_ic'] != 'N' or input_dics['fdsn_ic_auto'] == 'Y':
        create_tar_file_address = FDSN_ARC_IC(
            input_dics, clients=input_dics['fdsn_base_url'])
    if input_dics['arc_ic'] != 'N' or input_dics['arc_ic_auto'] == 'Y':
        create_tar_file_address = FDSN_ARC_IC(input_dics, clients='arc')
    # ------------------merge--------------------------------------
    if input_dics['fdsn_merge'] != 'N' or input_dics['fdsn_merge_auto'] == 'Y':
        FDSN_ARC_merge(input_dics, clients=input_dics['fdsn_base_url'])
    if input_dics['arc_merge'] != 'N' or input_dics['arc_merge_auto'] == 'Y':
        FDSN_ARC_merge(input_dics, clients='arc')
    # ------------------plot_tools--------------------------------------------
    if input_dics['plot_dir'].lower() != 'n':
        print 'Plotting %s' % input_dics['plot_dir']
        if input_dics['plot_all'] == 'Y' or input_dics['plot_fdsn'] == 'Y':
            plot_tools(input_dics, clients=input_dics['fdsn_base_url'])
        if input_dics['plot_arc'] == 'Y':
            plot_tools(input_dics, clients='arc')
    if input_dics['plot_all_events']:
        raw_input('\nPress enter to continue ..\n')
    # ------------------Compressing-------------------------------------------
    if create_tar_file_address:
        if input_dics['zip_w'] == 'Y' or input_dics['zip_r'] == 'Y':
            create_tar_file(input_dics, address=create_tar_file_address)
    # ------------------Email-------------------------------------------
    if input_dics['email'] != 'N':
        send_email(input_dics)
    return input_dics

########################################################################
########################################################################
########################################################################


def main():
    t1_pro = time.time()
    # Run the main program
    input_dics = obspyDMT()
    goodbye_printer(input_dics, t1_pro)
    # pass the return of main to the command line.
    sys.exit()

    # For debugging purposes:
    # from pycallgraph import PyCallGraph
    # from pycallgraph.output import GraphvizOutput
    # t1_pro = time.time()
    # graphviz = GraphvizOutput()
    # graphviz.output_file = 'profile_%s.png' % int(t1_pro)
    # with PyCallGraph(output=graphviz):
    #     # Run the main program
    #     input_dics = obspyDMT()
    #     goodbye_printer(input_dics, t1_pro)
    #     # pass the return of main to the command line.
    #     sys.exit()

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  plotxml_report.py
#   Purpose:   Plot the report generated by plotxml flag
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
import numpy as np
import sys

add_report = sys.argv[1]
report_fio = open(add_report, 'r')
report_fi = report_fio.readlines()

# --------------------------------------------------------------------
# collect information of good stations:
sta_lat_g = []
sta_lon_g = []
per_phase_diff_g = []
max_diff_g = []

# collect information of bad stations:
sta_lat_b = []
sta_lon_b = []
per_phase_diff_b = []
max_diff_b = []
time_wrong_b = []
id_b = []

sta_lat_time_shift = []
sta_lon_time_shift = []
time_shift = []

for i in range(1, len(report_fi)):
    line_report_fi = report_fi[i].split()
    al_corrected = float(line_report_fi[6]) - float(line_report_fi[7])
    if al_corrected != 0.:
        sta_lat_time_shift.append(float(line_report_fi[3]))
        sta_lon_time_shift.append(float(line_report_fi[4]))
        time_shift.append(al_corrected)
    if float(line_report_fi[1]) > 0. and abs(al_corrected) > 1.e-10:
        sta_lat_b.append(float(line_report_fi[3]))
        sta_lon_b.append(float(line_report_fi[4]))
        per_phase_diff_b.append(float(line_report_fi[1]))
        max_diff_b.append(float(line_report_fi[2]))
        id_b.append(line_report_fi[0])
        time_wrong_b.append(al_corrected)
        print report_fi[i].split('\n')[0]
    elif float(line_report_fi[1]) > 0. and \
                    int(float(line_report_fi[6])*1e5) == 0:
        sta_lat_b.append(float(line_report_fi[3]))
        sta_lon_b.append(float(line_report_fi[4]))
        per_phase_diff_b.append(float(line_report_fi[1]))
        max_diff_b.append(float(line_report_fi[2]))
        id_b.append(line_report_fi[0])
        time_wrong_b.append(al_corrected)
        print report_fi[i].split('\n')[0]
    elif float(line_report_fi[1]) > 0. and \
                    int(float(line_report_fi[7])*1e5) == 0:
        sta_lat_b.append(float(line_report_fi[3]))
        sta_lon_b.append(float(line_report_fi[4]))
        per_phase_diff_b.append(float(line_report_fi[1]))
        max_diff_b.append(float(line_report_fi[2]))
        id_b.append(line_report_fi[0])
        time_wrong_b.append(al_corrected)
        print report_fi[i].split('\n')[0]
    else:
        sta_lat_g.append(float(line_report_fi[3]))
        sta_lon_g.append(float(line_report_fi[4]))
        per_phase_diff_g.append(float(line_report_fi[1]))
        max_diff_g.append(float(line_report_fi[2]))

if len(sta_lat_g) > 0:
    # Plot GOOD stations
    plt.figure()
    m = Basemap(projection='robin', lon_0=0, lat_0=0)
    m.fillcontinents()
    parallels = np.arange(-90, 90, 30.)
    m.drawparallels(parallels, labels=[1, 1, 1, 1], fontsize=18)
    meridians = np.arange(-180., 180., 60.)
    m.drawmeridians(meridians, labels=[0, 0, 0, 1], fontsize=18)
    m.drawmapboundary()

    x, y = m(sta_lon_g, sta_lat_g)
    m.scatter(x, y, 20, c=np.array(per_phase_diff_g)*0., marker="o",
              edgecolor='none', zorder=10, cmap='gray', vmin=0., vmax=1.)
    plt.title('Good Stations', size=24, weight='bold')
    plt.savefig('compare_plots_good.png')

if len(sta_lat_time_shift) > 0:
    plt.figure()
    m = Basemap(projection='robin', lon_0=0, lat_0=0)
    m.fillcontinents()
    parallels = np.arange(-90, 90, 30.)
    m.drawparallels(parallels, labels=[1, 1, 1, 1], fontsize=18)
    meridians = np.arange(-180., 180., 60.)
    m.drawmeridians(meridians, labels=[0, 0, 0, 1], fontsize=18)
    m.drawmapboundary()

    x, y = m(sta_lon_time_shift, sta_lat_time_shift)
    m.scatter(x, y, 100, c=time_shift, marker="o", edgecolor='none',
              zorder=10, cmap='jet',
              vmin=min(time_shift), vmax=max(time_shift))
    cbar = plt.colorbar(orientation='horizontal', shrink=0.9)
    cbar.ax.tick_params(labelsize=18)
    plt.title('Time Shift', size=24, weight='bold')
    plt.savefig('time_shift.png')

if len(sta_lat_b) > 0:
    # Plot BAD stations
    plt.figure()
    m = Basemap(projection='robin', lon_0=0, lat_0=0)
    m.fillcontinents()
    parallels = np.arange(-90, 90, 30.)
    m.drawparallels(parallels, labels=[1, 1, 1, 1], fontsize=18)
    meridians = np.arange(-180., 180., 60.)
    m.drawmeridians(meridians, labels=[0, 0, 0, 1], fontsize=18)
    m.drawmapboundary()

    x, y = m(sta_lon_b, sta_lat_b)
    m.scatter(x, y, 100, c=per_phase_diff_b, marker="o", edgecolor='none',
              zorder=10, cmap='jet',
              vmin=min(per_phase_diff_b), vmax=100.)
    cbar = plt.colorbar(orientation='horizontal', shrink=0.9)
    cbar.ax.tick_params(labelsize=18)
    plt.title('Bad Stations (percentage)', size=24, weight='bold')
    plt.savefig('compare_plots_bad.png')

    plt.figure()
    m = Basemap(projection='robin', lon_0=0, lat_0=0)
    m.fillcontinents()
    parallels = np.arange(-90, 90, 30.)
    m.drawparallels(parallels, labels=[1, 1, 1, 1], fontsize=18)
    meridians = np.arange(-180., 180., 60.)
    m.drawmeridians(meridians, labels=[0, 0, 0, 1], fontsize=18)
    m.drawmapboundary()

    x, y = m(sta_lon_b, sta_lat_b)
    m.scatter(x, y, 100, c=time_wrong_b, marker="o", edgecolor='none',
              zorder=10, cmap='jet',
              vmin=min(time_wrong_b), vmax=max(time_wrong_b))
    cbar = plt.colorbar(orientation='horizontal', shrink=0.9)
    cbar.ax.tick_params(labelsize=18)
    plt.title('Bad Stations (time shift)', size=24, weight='bold')
    plt.savefig('compare_plots_bad_time.png')

    plt.figure()
    plt.hist(per_phase_diff_b, bins=1000)
    plt.xlabel('%Difference', size=24, weight='bold')
    plt.ylabel('#channels', size=24, weight='bold')
    plt.xticks(size=18, weight='bold')
    plt.yticks(size=18, weight='bold')
    plt.title('Difference (percentage)', size=24, weight='bold')

    plt.figure()
    plt.hist(max_diff_b, bins=1000)
    plt.xlabel('abs(maximum difference)', size=24, weight='bold')
    plt.ylabel('#channels', size=24, weight='bold')
    plt.xticks(size=18, weight='bold')
    plt.yticks(size=18, weight='bold')
    plt.title('Maximum Difference (abs)', size=24, weight='bold')

plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  update_handler.py
#   Purpose:   handling update mode in obspyDMT
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
import copy
from datetime import datetime
import os

from arclink_handler import ARC_available, ARC_waveform
from event_handler import quake_info
from fdsn_handler import FDSN_available, FDSN_waveform
from utility_codes import read_station_event, read_list_stas

import warnings
warnings.filterwarnings("ignore", category=DeprecationWarning)

# ##################### FDSN_update #####################################


def FDSN_update(input_dics, address):
    """
    Initialize directories and pass the required stations
    (removing the duplications) for FDSN update requests
    :param input_dics:
    :param address:
    :return:
    """
    print '\n*********************'
    print 'FDSN -- Updating Mode'
    print '*********************'
    t_update_1 = datetime.now()

    events, address_events = quake_info(address, 'info')

    for i in range(len(events)):
        if not input_dics['list_stas']:
            Stas_fdsn = FDSN_available(input_dics, events[i],
                                       address_events[i], event_number=i)
        else:
            Stas_fdsn = read_list_stas(input_dics['list_stas'],
                                       input_dics['normal_mode_syn'],
                                       input_dics['specfem3D'])

        if input_dics['fdsn_bulk'] != 'Y':
            print '\n%s-Availability for event: %s/%s ---> DONE' \
                  % (input_dics['fdsn_base_url'], i+1, len(events))
        else:
            print '\nFDSN-bulkfile for event: %s/%s ---> DONE' \
                  % (i+1, len(events))

        if Stas_fdsn != [[]]:
            Stas_req = rm_duplicate(Stas_fdsn,
                                    address=os.path.join(address_events[i]))
        else:
            Stas_req = None
            print '------------------------------'
            print 'There is no available station!'
            print '------------------------------'

        if not os.path.isdir(os.path.join(address_events[i], 'BH_RAW')):
            os.makedirs(os.path.join(address_events[i], 'BH_RAW'))

        if Stas_req:
            FDSN_waveform(input_dics, events, Stas_req, i, req_type='update')
        else:
            print '\nNo available station in %s for your request!' \
                  % input_dics['fdsn_base_url']
            print 'Check the next event...'
            continue

    print '\nTotal time for updating FDSN: %s' % (datetime.now() - t_update_1)

# ##################### ARC_update ######################################


def ARC_update(input_dics, address):
    """
    Initialize directories and pass the required stations
    (removing the duplications) for ArcLink update requests
    :param input_dics:
    :param address:
    :return:
    """
    print '\n************************'
    print 'ArcLink -- Updating Mode'
    print '************************'
    t_update_1 = datetime.now()

    events, address_events = quake_info(address, 'info')

    for i in range(len(events)):
        if not input_dics['list_stas']:
            Stas_arc = ARC_available(input_dics, events[i],
                                     address_events[i], event_number=i)
        else:
            Stas_arc = read_list_stas(input_dics['list_stas'],
                                      input_dics['normal_mode_syn'],
                                      input_dics['specfem3D'])

        print '\nArcLink-Availability for event: %s/%s ---> DONE' \
              % (i+1, len(events))

        if Stas_arc != [[]]:
            Stas_req = rm_duplicate(Stas_arc,
                                    address=os.path.join(address_events[i]))
        else:
            Stas_req = None
            print '------------------------------'
            print 'There is no available station!'
            print '------------------------------'

        if not os.path.isdir(os.path.join(address_events[i], 'BH_RAW')):
            os.makedirs(os.path.join(address_events[i], 'BH_RAW'))

        if Stas_req:
            ARC_waveform(input_dics, events, Stas_req, i, req_type='update')
        else:
            print '\nNo available station in ArcLink for your request!'
            print 'Check the next event...'
            continue

    print '\nTotal time for updating ArcLink: %s' \
          % (datetime.now() - t_update_1)

# ##################### rm_duplicate ####################################


def rm_duplicate(all_sta_avail, address):
    """
    remove duplicates and give back the required list for updating
    """

    id_avai_stas = []
    for sta in all_sta_avail:
        if sta[2] == '--' or sta[2] == '  ':
            sta[2] = ''
        if len(sta) == 7:
            id_avai_stas.append('%s_%s_%s_%s_%s_%s_%s'
                                % (sta[0], sta[1], sta[2], sta[3],
                                   sta[4], sta[5], sta[6]))
        elif len(sta) == 8:
            id_avai_stas.append('%s_%s_%s_%s_%s_%s_%s_%s'
                                % (sta[0], sta[1], sta[2], sta[3],
                                   sta[4], sta[5], sta[6], sta[7]))

    sta_ev_saved = read_station_event(address)

    id_all_saved_stas = []
    for saved_sta in sta_ev_saved[0]:
        id_all_saved_stas.append('%s_%s_%s_%s' % (saved_sta[0], saved_sta[1],
                                                  saved_sta[2], saved_sta[3]))

    stas_update = copy.deepcopy(id_avai_stas)
    del_num = []
    for saved_sta in id_all_saved_stas:
        for j in range(len(stas_update)):
            if saved_sta in stas_update[j]:
                del_num.append(j)

    del_num.sort(reverse=True)
    for dn in del_num:
        del stas_update[dn]

    for i in range(len(stas_update)):
        stas_update[i] = stas_update[i].split('_')

    stas_update.sort()
    print '------------------------------------------'
    print 'Info:'
    print 'Number of all saved stations:     %s' % len(id_all_saved_stas)
    print 'Number of all available stations: %s' % len(id_avai_stas)
    print 'Number of stations to update for: %s' % len(stas_update)
    print '------------------------------------------'

    return stas_update


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  format_converter.py
#   Purpose:   handling seismic data format
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
import os
from obspy.core import read

from utility_codes import read_station_event

import warnings
warnings.filterwarnings("ignore", category=DeprecationWarning)

# ##################### writesac_all ####################################


def writesac_all(i, address_events):
    """
    Change the format of trace(s) to SAC and fill in the header information
    :param i:
    :param address_events:
    :return:
    """
    sta_ev = read_station_event(address_events[i])
    ls_saved_stas = []

    for j in range(len(sta_ev[0])):
        station_id = '%s.%s.%s.%s' % (sta_ev[0][j][0],
                                      sta_ev[0][j][1],
                                      sta_ev[0][j][2],
                                      sta_ev[0][j][3])
        ls_saved_stas.append(os.path.join(address_events[i],
                                          'BH_RAW', station_id))
    for j in range(len(sta_ev[0])):
        try:
            st = read(ls_saved_stas[j])
            if len(st) > 1:
                print "WARNING: %s\nprobably has some gaps" % ls_saved_stas[j]
                print "\nIt will be merged (fill_value=0) before " \
                      "writing the SAC file!"
                st.merge(method=1, fill_value=0, interpolation_samples=0)
            st[0].write(ls_saved_stas[j], format='SAC')
            tr = read(ls_saved_stas[j])[0]
            if sta_ev[0][j][4]:
                tr.stats.sac.stla = float(sta_ev[0][j][4])
            if sta_ev[0][j][5]:
                tr.stats.sac.stlo = float(sta_ev[0][j][5])
            if sta_ev[0][j][6]:
                tr.stats.sac.stel = float(sta_ev[0][j][6])
            if sta_ev[0][j][7]:
                tr.stats.sac.stdp = float(sta_ev[0][j][7])

            if sta_ev[0][j][9]:
                tr.stats.sac.evla = float(sta_ev[0][j][9])
            if sta_ev[0][j][10]:
                tr.stats.sac.evlo = float(sta_ev[0][j][10])
            if sta_ev[0][j][11]:
                tr.stats.sac.evdp = float(sta_ev[0][j][11])
            if sta_ev[0][j][12]:
                tr.stats.sac.mag = float(sta_ev[0][j][12])

            tr.write(ls_saved_stas[j], format='SAC')

        except Exception as e:
            print '\nWARNING: %s' % e
            print ls_saved_stas[j]
            print '------------------'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  plotting_tools.py
#   Purpose:   Collection of plotting tools
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
import fnmatch
import glob
import matplotlib.pyplot as plt
from mpl_toolkits.basemap import Basemap
import numpy as np
from obspy import read_inventory
from obspy.signal import pazToFreqResp
try:
    from obspy.geodetics import locations2degrees
except Exception, e:
    from obspy.core.util import locations2degrees
from obspy.core import read, UTCDateTime
from obspy.imaging.beachball import Beach
import os
import sys

from event_handler import quake_info
from utility_codes import read_station_event, read_event_dic

import warnings
warnings.filterwarnings("ignore", category=DeprecationWarning)

# ##################### plot_tools ############################################


def plot_tools(input_dics, clients, all_events=False, all_stations=False):
    """
    Managing the required inputs for plotting tools
    :param input_dics:
    :param clients:
    :return:
    """
    if input_dics['plot_dir'].lower() != 'n':
        events, address_events = quake_info(input_dics['plot_dir'], 'info')
    else:
        sys.exit('ERROR: --plot_dir has not set!')

    ls_saved_stas = []
    ls_add_stas = []
    for i in range(len(events)):
        ls_saved_stas_tmp = []
        ls_add_stas_tmp = []
        sta_ev = read_station_event(address_events[i])
        event_dic = read_event_dic(address_events[i])

        pass_ev = True
        if not all_events:
            pass_ev = plot_filter_event(input_dics, event_dic)
        if not pass_ev:
            continue
        for j in range(len(sta_ev[0])):
            if input_dics['plot_type'].lower() == 'raw':
                BH_file = 'BH_RAW'
                network = sta_ev[0][j][0]
            elif input_dics['plot_type'].lower() == 'corrected':
                if input_dics['corr_unit'].lower() == 'dis':
                    BH_file = 'BH'
                    network = 'dis.%s' % sta_ev[0][j][0]
                elif input_dics['corr_unit'].lower() == 'vel':
                    BH_file = 'BH_VEL'
                    network = 'vel.%s' % sta_ev[0][j][0]
                elif input_dics['corr_unit'].lower() == 'acc':
                    BH_file = 'BH_ACC'
                    network = 'acc.%s' % sta_ev[0][j][0]

            pass_sta = True
            if not all_stations:
                pass_sta = plot_filter_station(input_dics, sta_ev[0][j])
            if not pass_sta:
                continue
            station_id = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s' \
                         % (network, sta_ev[0][j][1], sta_ev[0][j][2],
                            sta_ev[0][j][3], sta_ev[0][j][4],
                            sta_ev[0][j][5], sta_ev[0][j][6],
                            sta_ev[0][j][7], sta_ev[0][j][8],
                            sta_ev[0][j][9], sta_ev[0][j][10],
                            sta_ev[0][j][11], sta_ev[0][j][12],
                            sta_ev[0][j][13])

            if input_dics['plot_all'] != 'Y':
                if clients == sta_ev[0][j][13]:
                    station_id = station_id.split(',')
                    if event_dic['focal_mechanism']:
                        station_id.extend(event_dic['focal_mechanism'])
                    else:
                        station_id.extend('False')
                    ls_saved_stas_tmp.append(station_id)
                    ls_add_stas_tmp.append(
                        os.path.join(address_events[i], BH_file,
                                     '%s.%s.%s.%s'
                                     % (network, sta_ev[0][j][1],
                                        sta_ev[0][j][2],
                                        sta_ev[0][j][3])))

            elif input_dics['plot_all'] == 'Y':
                station_id = station_id.split(',')
                if event_dic['focal_mechanism']:
                    station_id.extend(event_dic['focal_mechanism'])
                else:
                    station_id.extend('False')
                ls_saved_stas_tmp.append(station_id)
                ls_add_stas_tmp.append(
                    os.path.join(address_events[i], BH_file,
                                 '%s.%s.%s.%s'
                                 % (network, sta_ev[0][j][1],
                                    sta_ev[0][j][2], sta_ev[0][j][3])))

        ls_saved_stas.append(ls_saved_stas_tmp)
        ls_add_stas.append(ls_add_stas_tmp)

    if input_dics['plot_ray_gmt']:
        plot_ray_gmt(input_dics, ls_saved_stas)
    else:
        for k in ['plot_sta', 'plot_ev', 'plot_ray']:
            if input_dics[k]:
                plot_sta_ev_ray(input_dics, ls_saved_stas)
                break

    if input_dics['plot_epi']:
        plot_epi(input_dics, ls_add_stas, ls_saved_stas)

    if input_dics['plot_dt']:
        plot_dt(input_dics, address_events)

# ##################### plot_filter_event ###############################


def plot_filter_event(input_dics, event_dic):
    """
    check whether the event can pass the criteria
    :param input_dics:
    :param event_dic:
    :return:
    """
    if not event_dic['datetime'] <= UTCDateTime(input_dics['max_date']):
        return False
    if not event_dic['datetime'] >= UTCDateTime(input_dics['min_date']):
        return False
    if not event_dic['magnitude'] <= float(input_dics['max_mag']):
        return False
    if not event_dic['magnitude'] >= float(input_dics['min_mag']):
        return False
    if not event_dic['depth'] <= float(input_dics['max_depth']):
        return False
    if not event_dic['depth'] >= float(input_dics['min_depth']):
        return False
    if isinstance(input_dics['evlatmin'], float):
        if not event_dic['latitude'] <= float(input_dics['evlatmax']):
            return False
        if not event_dic['longitude'] <= float(input_dics['evlonmax']):
            return False
        if not event_dic['latitude'] >= float(input_dics['evlatmin']):
            return False
        if not event_dic['longitude'] >= float(input_dics['evlonmin']):
            return False
    return True

# ##################### plot_filter_station ###############################


def plot_filter_station(input_dics, sta_ev):
    """
    check whether the station can pass the criteria
    :param input_dics:
    :param sta_ev:
    :return:
    """
    if not fnmatch.fnmatch(sta_ev[0], input_dics['net']):
        return False
    if not fnmatch.fnmatch(sta_ev[1], input_dics['sta']):
        return False
    if not fnmatch.fnmatch(sta_ev[2], input_dics['loc']):
        return False
    if not fnmatch.fnmatch(sta_ev[3], input_dics['cha']):
        return False
    if isinstance(input_dics['mlat_rbb'], float):
        if not float(sta_ev[4]) <= input_dics['Mlat_rbb']:
            return False
        if not float(sta_ev[4]) >= input_dics['mlat_rbb']:
            return False
        if not float(sta_ev[5]) <= input_dics['Mlon_rbb']:
            return False
        if not float(sta_ev[5]) >= input_dics['mlon_rbb']:
            return False
    return True

# ##################### plot_sta_ev_ray ###############################


def plot_sta_ev_ray(input_dics, ls_saved_stas):
    """
    Plots stations, events and ray paths on a map with basemap.
    :param input_dics:
    :param ls_saved_stas:
    :return:
    """

    plt.figure(figsize=(20., 10.))
    plt_stations = input_dics['plot_sta']
    plt_events = input_dics['plot_ev']
    plt_ray_path = input_dics['plot_ray']

    if input_dics['evlatmin'] is None:
        evlatmin = -90
        evlatmax = +90
        evlonmin = -180
        evlonmax = +180
        glob_map = True
    else:
        evlatmin = input_dics['evlatmin']
        evlatmax = input_dics['evlatmax']
        evlonmin = input_dics['evlonmin']
        evlonmax = input_dics['evlonmax']
        glob_map = False

    if plt_ray_path:
        # # hammer, kav7, cyl, mbtfpq, moll
        # m = Basemap(projection='aeqd', lon_0=0, lat_0=0)
        # parallels = np.arange(-90, 90, 30.)
        # m.drawparallels(parallels, color='gray')
        # meridians = np.arange(-180., 180., 60.)
        # m.drawmeridians(meridians, color='gray')
        m = Basemap(projection='robin', lon_0=180)
        parallels = np.arange(-90, 90, 30.)
        m.drawparallels(parallels, labels=[1, 1, 1, 1], fontsize=24)
        meridians = np.arange(-180., 180., 60.)
        m.drawmeridians(meridians, labels=[1, 1, 1, 1], fontsize=24)
        width_beach = 10e5
        width_station = 50
    elif not glob_map:
        m = Basemap(projection='cyl', llcrnrlat=evlatmin, urcrnrlat=evlatmax,
                    llcrnrlon=evlonmin, urcrnrlon=evlonmax)
        parallels = np.arange(-90, 90, 5.)
        m.drawparallels(parallels, labels=[1, 0, 0, 1], fontsize=12)
        meridians = np.arange(-180., 180., 5.)
        m.drawmeridians(meridians, labels=[1, 0, 0, 1], fontsize=12)
        width_beach = 5
        width_station = 10
    elif glob_map:
        # hammer, kav7, cyl, mbtfpq, moll
        m = Basemap(projection='robin', lon_0=180)
        parallels = np.arange(-90, 90, 30.)
        m.drawparallels(parallels, labels=[1, 1, 1, 1], fontsize=24)
        meridians = np.arange(-180., 180., 60.)
        m.drawmeridians(meridians, labels=[1, 1, 1, 1], fontsize=24)
        width_beach = 5e5
        width_station = 50
    else:
        sys.exit('ERROR: %s' % input_dics)

    raw_input_resp = raw_input('Choose your map style:\n'
                               '1. Bluemarble (PIL should be installed)\n'
                               '2. Etopo (PIL should be installed)\n'
                               '3. Shaderelief (PIL should be installed)\n'
                               '4. Simple\n')
    if raw_input_resp == '1':
        m.bluemarble(scale=0.5)
    elif raw_input_resp == '2':
        m.etopo(scale=0.5)
    elif raw_input_resp == '3':
        m.shadedrelief(scale=0.1)
    else:
        m.fillcontinents()

    len_src_rcv = 0
    for i in range(len(ls_saved_stas)):
        len_src_rcv += len(ls_saved_stas[i])
    print 'Length src-rcv pairs: %s' % len_src_rcv 
    if plt_events:
        for i in range(len(ls_saved_stas)):
            if input_dics['plot_focal']:
                if ls_saved_stas[i][0][14] == 'F':
                    sys.exit('ERROR:\nMoment tensor does not exists!')
                x, y = m(float(ls_saved_stas[i][0][10]),
                         float(ls_saved_stas[i][0][9]))
                focmecs = [float(ls_saved_stas[i][0][14]),
                           float(ls_saved_stas[i][0][15]),
                           float(ls_saved_stas[i][0][16]),
                           float(ls_saved_stas[i][0][17]),
                           float(ls_saved_stas[i][0][18]),
                           float(ls_saved_stas[i][0][19])]
                ax = plt.gca()
                b = Beach(focmecs, xy=(x, y), facecolor='blue',
                          width=width_beach, linewidth=1, alpha=0.85)
                b.set_zorder(10)
                ax.add_collection(b)
            else:
                x, y = m(float(ls_saved_stas[i][0][10]),
                         float(ls_saved_stas[i][0][9]))
                magnitude = float(ls_saved_stas[i][0][12])
                m.scatter(x, y, color="blue", s=10*magnitude,
                          edgecolors='none', marker="o",
                          zorder=5, alpha=0.6)
    else:
        print 'No events will be plotted.'

    if plt_stations:
        for i in range(len(ls_saved_stas)):
            for j in range(len(ls_saved_stas[i])):
                x, y = m(float(ls_saved_stas[i][j][5]),
                         float(ls_saved_stas[i][j][4]))
                m.scatter(x, y, color='red', s=width_station,
                          edgecolors='none', marker='v',
                          zorder=4, alpha=0.9)
    else:
        print 'No stations will be plotted on your map!'

    if plt_ray_path:
        for i in range(len(ls_saved_stas)):
            print 'Projecting Event: %s/%s' % (i+1, len(ls_saved_stas))
            for j in range(len(ls_saved_stas[i])):
                gcline = m.drawgreatcircle(
                    float(ls_saved_stas[i][j][10]),
                    float(ls_saved_stas[i][j][9]),
                    float(ls_saved_stas[i][j][5]),
                    float(ls_saved_stas[i][j][4]),
                    color='k', alpha=0.0)
                gcx, gcy = gcline[0].get_data()
                gcx_diff = gcx[0:-1] - gcx[1:]
                gcy_diff = gcy[0:-1] - gcy[1:]
                if np.max(abs(gcx_diff))/abs(gcx_diff[0]) > 800:
                    gcx_max_arg = abs(gcx_diff).argmax()
                    plt.plot(gcx[0:gcx_max_arg], gcy[0:gcx_max_arg],
                             color='k', alpha=0.2)
                    plt.plot(gcx[gcx_max_arg+1:], gcy[gcx_max_arg+1:],
                             color='k', alpha=0.2)
                elif np.max(abs(gcy_diff))/abs(gcy_diff[0]) > 400:
                    gcy_max_arg = abs(gcy_diff).argmax()
                    plt.plot(gcy[0:gcy_max_arg], gcy[0:gcy_max_arg],
                             color='k', alpha=0.2)
                    plt.plot(gcy[gcy_max_arg+1:], gcy[gcy_max_arg+1:],
                             color='k', alpha=0.2)
                else:
                    m.drawgreatcircle(
                        float(ls_saved_stas[i][j][10]),
                        float(ls_saved_stas[i][j][9]),
                        float(ls_saved_stas[i][j][5]),
                        float(ls_saved_stas[i][j][4]),
                        color='k', alpha=0.2)
    else:
        print 'No rays path will be plotted on your map!'

    plt.savefig(os.path.join(input_dics['plot_save'],
                             'ev_sta_ray.%s' % input_dics['plot_format']))
    plt.show()

# ##################### plot_ray_gmt ####################################


def plot_ray_gmt(input_dics, ls_saved_stas):
    """
    Plot: stations, events and ray paths for the specified directory using GMT
    :param input_dics:
    :param ls_saved_stas:
    :return:
    """
    if input_dics['plot_focal']:
        with open('./gmt_psmeca.txt', 'w') as outfile:
            outfile.write('lon lat depth mrr mtt mpp mrt mrp mtp iexp name \n')
            for i in range(len(ls_saved_stas)):
                if not ls_saved_stas[i][0][14]:
                    sys.exit('ERROR:\nMoment tensor does not exists!')
                ev_lon = ls_saved_stas[i][0][10]
                ev_lat = ls_saved_stas[i][0][9]
                ev_dep = ls_saved_stas[i][0][11]
                ev_mrr = float(ls_saved_stas[i][0][14])
                ev_mtt = float(ls_saved_stas[i][0][15])
                ev_mpp = float(ls_saved_stas[i][0][16])
                ev_mrt = float(ls_saved_stas[i][0][17])
                ev_mrp = float(ls_saved_stas[i][0][18])
                ev_mtp = float(ls_saved_stas[i][0][19])

                scalar_moment = np.sqrt(np.power(ev_mrr, 2) +
                                        np.power(ev_mtt, 2) +
                                        np.power(ev_mpp, 2) +
                                        np.power(ev_mrt, 2) +
                                        np.power(ev_mrp, 2) +
                                        np.power(ev_mtp, 2))/np.sqrt(2)

                ev_mom_max = max(abs(ev_mrr),
                                 abs(ev_mtt),
                                 abs(ev_mpp),
                                 abs(ev_mrt),
                                 abs(ev_mrp),
                                 abs(ev_mtp))
                ev_mrr_fl = format(ev_mrr/ev_mom_max, 'f')
                ev_mtt_fl = format(ev_mtt/ev_mom_max, 'f')
                ev_mpp_fl = format(ev_mpp/ev_mom_max, 'f')
                ev_mrt_fl = format(ev_mrt/ev_mom_max, 'f')
                ev_mrp_fl = format(ev_mrp/ev_mom_max, 'f')
                ev_mtp_fl = format(ev_mtp/ev_mom_max, 'f')

                outfile.write('%s  %s  %s  %s  %s  %s  %s  %s  %s  %s\n'
                              % (ev_lon, ev_lat, ev_dep,
                                 ev_mrr_fl,
                                 ev_mtt_fl,
                                 ev_mpp_fl,
                                 ev_mrt_fl,
                                 ev_mrp_fl,
                                 ev_mtp_fl,
                                 str(scalar_moment).split('e')[1][1:]))
    else:
        with open('./gmt_events.txt', 'w') as outfile:
            for i in range(len(ls_saved_stas)):
                ev_lon = ls_saved_stas[i][0][10]
                ev_lat = ls_saved_stas[i][0][9]
                outfile.write('%s   %s \n' % (ev_lon, ev_lat))
    outfile.close()

    with open('./gmt_sta_ev_path.txt', 'w') as outfile:
        for i in range(len(ls_saved_stas)):
            for j in range(len(ls_saved_stas[i])):
                ev_lon = ls_saved_stas[i][j][10]
                ev_lat = ls_saved_stas[i][j][9]
                sta_lon = ls_saved_stas[i][j][5]
                sta_lat = ls_saved_stas[i][j][4]
                outfile.write('%s   %s \n %s   %s\n'
                              % (ev_lon, ev_lat,
                                 sta_lon, sta_lat))
    outfile.close()

    with open('./gmt_station.txt', 'w') as outfile:
        for i in range(len(ls_saved_stas)):
            for j in range(len(ls_saved_stas[i])):
                sta_lon = ls_saved_stas[i][j][5]
                sta_lat = ls_saved_stas[i][j][4]
                outfile.write('%s   %s \n' % (sta_lon, sta_lat))
    outfile.close()

    # GMT part:
    os.system('psbasemap -Rd -JK180/9i -B45g30 -K -Xc -Yc> gmt_output.ps')

    os.system('pscoast -Rd -JK180/9i -B45g30:."World-wide Ray Path Coverage": '
              '-Dc -A1000 -Glightgray -Wthinnest -t0 -O -K >> gmt_output.ps')

    os.system('psxy ./gmt_sta_ev_path.txt -JK180/9i -Rd -O -K -W0.1,'
              'black -t20  >> gmt_output.ps')
    os.system('psxy ./gmt_station.txt -JK180/9i -Rd -Si0.2c -GRed -O -K >> '
              'gmt_output.ps')
    if input_dics['plot_focal']:
        os.system('psmeca gmt_psmeca.txt -h1 -JK180/9i -Rd -W1 -GBlue '
                  '-Sd1.0  -O >> gmt_output.ps')
    else:
        os.system('psxy ./gmt_events.txt -JK180/9i -Rd -Sc0.2c '
                  '-GBlue -O >> gmt_output.ps')

    os.system('ps2raster gmt_output.ps -A -P -Tf')

    os.system('mv gmt_output.ps ray_coverage_gmt.ps')
    os.system('mv gmt_output.pdf ray_coverage_gmt.pdf')

    os.system('xdg-open ray_coverage_gmt.pdf')

# ##################### plot_epi ########################################


def plot_epi(input_dics, ls_add_stas, ls_saved_stas):
    """
    Plot arranged waveforms by epicentral distance versus time
    :param input_dics:
    :param ls_add_stas:
    :param ls_saved_stas:
    :return:
    """

    for target in range(len(ls_add_stas)):
        plt.clf()
        plt.figure(figsize=(20., 10.))

        for i in range(len(ls_add_stas[target])):
            try:
                tr = read(ls_add_stas[target][i])[0]
                tr.normalize()
                dist = locations2degrees(float(ls_saved_stas[target][i][9]),
                                         float(ls_saved_stas[target][i][10]),
                                         float(ls_saved_stas[target][i][4]),
                                         float(ls_saved_stas[target][i][5]))
                if input_dics['min_epi'] <= dist <= input_dics['max_epi']:
                    plt.plot(
                        np.linspace(0,
                                    (tr.stats.npts-1)/tr.stats.sampling_rate,
                                    tr.stats.npts), tr.data + dist,
                        color='black', lw=2)
            except Exception as e:
                print 'WARNING: %s' % e
                pass
            plt.xlabel('Time (sec)', size=24, weight='bold')
            plt.ylabel('Epicentral distance (deg)', size=24, weight='bold')
            plt.xticks(size=18, weight='bold')
            plt.yticks(size=18, weight='bold')

        print os.path.join(input_dics['plot_save'],
                           'epi_time_%s.%s'
                           % (ls_saved_stas[target][0][8],
                              input_dics['plot_format']))
        plt.savefig(os.path.join(input_dics['plot_save'],
                                 'epi_time_%s.%s'
                                 % (ls_saved_stas[target][0][8],
                                    input_dics['plot_format'])))

# ##################### plot_xml_response ###############################


def plot_xml_response(input_dics):
    """
    This function plots the response file of stationXML file(s)
    It has several modes such as:
    plotting all the stages
    plotting full response file
    plotting selected stages
    plotting only PAZ
    :param input_dics:
    :return:
    """

    print '[INFO] plotting StationXML file/files in: %s' % \
          input_dics['plotxml_dir']
    if not os.path.isdir('./stationxml_plots'):
        print '[INFO] creating stationxml_plots directory...',
        os.mkdir('./stationxml_plots')
        print 'DONE'

    # Assign the input_dics parameters to the running parameters
    # in this function:
    stxml_dir = input_dics['plotxml_dir']
    plotxml_datetime = input_dics['plotxml_date']
    min_freq = input_dics['plotxml_min_freq']
    output = input_dics['plotxml_output']
    if output.lower() == 'dis':
        output = 'DISP'
    start_stage = input_dics['plotxml_start_stage']
    end_stage_input = input_dics['plotxml_end_stage']
    percentage = input_dics['plotxml_percentage']/100.
    threshold = input_dics['plotxml_phase_threshold']
    plot_response = input_dics['plotxml_response']
    plotstage12 = input_dics['plotxml_plotstage12']
    plotpaz = input_dics['plotxml_paz']
    plotallstages = input_dics['plotxml_allstages']
    plot_map_compare = input_dics['plotxml_map_compare']

    if os.path.isfile(stxml_dir):
        addxml_all = glob.glob(os.path.join(stxml_dir))
    elif os.path.isdir(stxml_dir):
        addxml_all = glob.glob(os.path.join(stxml_dir, 'STXML.*'))
    else:
        try:
            addxml_all = glob.glob(os.path.join(stxml_dir))
        except Exception, e:
            print 'ERROR: %s' % e
            sys.exit('[ERROR] wrong address: %s' % stxml_dir)

    addxml_all.sort()

    sta_lat = []
    sta_lon = []
    latlon_color = []
    report_fio = open(os.path.join('./stationxml_plots',
                                   'report_stationxml'), 'w')
    report_fio.writelines('channel_id\t\t\t\t%(Phase)\t\t'
                          'Max Diff(abs) \tLat\t\t\tLon\t\t\tDatetime\t'
                          'decimation delay\tdecimation correction\n')
    report_fio.close()
    add_counter = 0
    for addxml in addxml_all:
        end_stage = end_stage_input
        add_counter += 1
        print 40*'-'
        print '%s/%s' % (add_counter, len(addxml_all))
        try:
            xml_inv = read_inventory(addxml, format='stationXML')
            print "[STATIONXML] %s" % addxml
            cha_name = xml_inv.get_contents()['channels'][0]
            if plotxml_datetime:
                cha_date = plotxml_datetime
            else:
                cha_date = xml_inv.networks[0][0][-1].start_date
                print '[INFO] plotxml_date has not been set, the start_date ' \
                      'of the last channel in stationXML file will be used ' \
                      'instead: %s' % cha_date

            xml_response = xml_inv.get_response(cha_name, cha_date + 0.1)
            for stage in xml_response.response_stages[::-1]:
                if (stage.decimation_input_sample_rate is not None
                        and stage.decimation_factor is not None):
                    sampling_rate = (stage.decimation_input_sample_rate /
                                     stage.decimation_factor)
                    break

            t_samp = 1.0 / sampling_rate
            nyquist = sampling_rate / 2.0
            nfft = int(sampling_rate / min_freq)

            end_stage = min(len(xml_response.response_stages), end_stage)

            if plotallstages:
                plot_xml_plotallstages(xml_response, t_samp, nyquist, nfft,
                                       min_freq, output,
                                       start_stage, end_stage, cha_name)

            try:
                cpx_response, freq = xml_response.get_evalresp_response(
                    t_samp=t_samp, nfft=nfft, output=output,
                    start_stage=start_stage, end_stage=end_stage)
                cpx_12, freq = xml_response.get_evalresp_response(
                    t_samp=t_samp, nfft=nfft, output=output, start_stage=1,
                    end_stage=2)
            except Exception, e:
                print 'WARNING: %s' % e
                continue

            paz, decimation_delay, decimation_correction = \
                convert_xml_paz(xml_response, output, cha_name, cha_date)
            if not paz:
                continue
            h, f = pazToFreqResp(paz['poles'], paz['zeros'], paz['gain'],
                                 1./sampling_rate, nfft, freq=True)

            phase_resp = np.angle(cpx_response)
            phase_12 = np.angle(cpx_12)

            if plot_response:
                plt.close()
                plt.ion()
                plt.figure(figsize=(20, 10))
                plt.suptitle(cha_name, size=24, weight='bold')
                if plotpaz or plotstage12:
                    plt.subplot(2, 2, 1)
                else:
                    plt.subplot(2, 1, 1)
                plt.loglog(freq, abs(cpx_response), color='blue',
                           lw=3, label='full-resp')
                if plotstage12:
                    plt.loglog(freq, abs(cpx_12), ls='--', color='black',
                               lw=3, label='Stage1,2')
                if plotpaz:
                    plt.loglog(f, abs(h)*paz['sensitivity'], color='red',
                               lw=3, label='PAZ')
                plt.axvline(nyquist, ls="--", color='blue', lw=3)
                plt.ylabel('Amplitude', size=24, weight='bold')
                plt.xticks(size=18, weight='bold')
                plt.yticks(size=18, weight='bold')
                plt.xlim(xmin=min_freq, xmax=nyquist+5)
                plt.ylim(ymax=max(1.2*abs(cpx_response)))
                plt.legend(loc=0, prop={'size': 18, 'weight': 'bold'})
                plt.grid()

                if plotpaz or plotstage12:
                    plt.subplot(2, 2, 3)
                else:
                    plt.subplot(2, 1, 2)
                plt.semilogx(freq, phase_resp, color='blue',
                             lw=3, label='full-resp')
                if plotstage12:
                    plt.semilogx(freq, phase_12, ls='--', color='black',
                                 lw=3, label='Stage1,2')
                if plotpaz:
                    plt.semilogx(f, np.angle(h), color='red',
                                 lw=3, label='PAZ')
                plt.axvline(nyquist, ls="--", color='blue', lw=3)
                plt.xlabel('Frequency [Hz]', size=24, weight='bold')
                plt.ylabel('Phase [rad]', size=24, weight='bold')
                plt.xticks(size=18, weight='bold')
                plt.yticks(size=18, weight='bold')
                plt.xlim(xmin=min_freq, xmax=nyquist+5)
                plt.legend(loc=0, prop={'size': 18, 'weight': 'bold'})
                plt.grid()

                if plotstage12 or plotpaz:
                    plt.subplot(2, 2, 2)
                    if plotstage12:
                        plt.loglog(freq, abs(abs(cpx_response) - abs(cpx_12)),
                                   '--', color='black', lw=3,
                                   label='|full-resp - Stage1,2|')
                    if plotpaz:
                        plt.loglog(f, abs(abs(cpx_response) -
                                          abs(h)*paz['sensitivity']),
                                   color='red', lw=3,
                                   label='|full-resp - PAZ|')
                    plt.axvline(nyquist, ls="--", color='blue', lw=3)
                    plt.axvline(percentage*nyquist, ls="--",
                                color='blue', lw=3)
                    plt.ylabel('Amplitude Difference', size=24, weight='bold')
                    plt.xticks(size=18, weight='bold')
                    plt.yticks(size=18, weight='bold')
                    plt.xlim(xmin=min_freq, xmax=nyquist+5)
                    plt.ylim(ymax=max(1.2*abs(cpx_response)))
                    plt.legend(loc=0, prop={'size': 18, 'weight': 'bold'})
                    plt.grid()

                    plt.subplot(2, 2, 4)
                    if plotstage12:
                        plt.semilogx(freq, abs(phase_resp - phase_12),
                                     color='black', ls='--', lw=3,
                                     label='|full-resp - Stage1,2|')
                    if plotpaz:
                        plt.semilogx(freq, abs(phase_resp - np.angle(h)),
                                     color='red', lw=3,
                                     label='|full-resp - PAZ|')
                    plt.axvline(nyquist, ls="--", color='blue', lw=3)
                    plt.axvline(percentage*nyquist, ls="--",
                                color='blue', lw=3)
                    plt.xlabel('Frequency [Hz]', size=24, weight='bold')
                    plt.ylabel('Phase Difference [rad]',
                               size=24, weight='bold')
                    plt.xticks(size=18, weight='bold')
                    plt.yticks(size=18, weight='bold')
                    plt.xlim(xmin=min_freq, xmax=nyquist+5)
                    plt.legend(loc=0, prop={'size': 18, 'weight': 'bold'})
                    plt.grid()
                plt.savefig(os.path.join('stationxml_plots',
                                         cha_name + '.png'))

            # compare = abs(phase[:int(0.8*len(phase))] -
            #              np.angle(h[:int(0.8*len(phase))]))
            # if len(compare[compare>0.1]) > 0:
            #    lat_red.append(xml_inv.get_coordinates(cha_name)['latitude'])
            #    lon_red.append(xml_inv.get_coordinates(cha_name)['longitude'])
            #    print cha_name
            #    print paz
            # else:
            #    lat_blue.append(xml_inv.get_coordinates(cha_name)['latitude'])
            #    lon_blue.append(xml_inv.get_coordinates(cha_name)['longitude'])

            phase_resp_check = phase_resp[:int(percentage*len(phase_resp))]
            phase_h_check = np.angle(h)[:int(percentage*len(np.angle(h)))]
            if not len(phase_resp_check) == len(phase_h_check):
                sys.exit('Lengths of phase responses do not match: '
                         '%s (StationXML) != %s (PAZ)'
                         % (len(phase_resp_check), len(phase_h_check)))
            compare = abs(phase_resp_check - phase_h_check)
            percent_compare = \
                float(len(compare[compare >= 0.05]))/len(compare)*100
            latlondep = get_coordinates(xml_inv.networks[0],
                                        cha_name,
                                        cha_date + 0.1)
            sta_lat.append(latlondep['latitude'])
            sta_lon.append(latlondep['longitude'])

            d_c = np.sum(decimation_delay) - np.sum(decimation_correction)
            if not d_c == 0:
                latlon_color.append(0.)
            else:
                latlon_color.append(percent_compare)

            if percent_compare >= threshold:
                plot_xml_plotallstages(xml_response, t_samp, nyquist, nfft,
                                       min_freq, output,
                                       start_stage, end_stage, cha_name)
            report_fio = open(os.path.join('./stationxml_plots',
                                           'report_stationxml'), 'a')
            report_fio.writelines(
                '%s\t\t\t%6.2f\t\t\t%6.2f\t\t\t%6.2f\t\t%7.2f\t\t%s\t%s\t%s\n'
                % (cha_name,
                   round(percent_compare, 2),
                   round(max(abs(compare)), 2),
                   round(sta_lat[-1], 2),
                   round(sta_lon[-1], 2),
                   cha_date,
                   np.sum(decimation_delay),
                   np.sum(decimation_correction)))
            report_fio.close()
        except Exception, e:
            print 'Exception: %s' % e

    if plot_map_compare:
        plt.figure()
        m = Basemap(projection='robin', lon_0=0, lat_0=0)
        # m.drawcoastlines()
        m.fillcontinents()
        m.drawparallels(np.arange(-90., 120., 30.))
        m.drawmeridians(np.arange(0., 420., 60.))
        m.drawmapboundary()

        x, y = m(sta_lon, sta_lat)
        m.scatter(x, y, 100, c=latlon_color, marker="v",
                  edgecolor='none', zorder=10, cmap='hot_r')
        plt.colorbar(orientation='horizontal')
        plt.savefig(os.path.join('stationxml_plots', 'compare_plots.png'))
        plt.show()
        raw_input('Press Enter...')
    sys.exit('[EXIT] obspyDMT finished normally...')

# ##################### get_coordinates ##########################


def get_coordinates(xml_network, seed_id, datetime=None):
    """
    FROM OBSPY WEBPAGE!

    Return coordinates for a given channel.

    :type seed_id: str
    :param seed_id: SEED ID string of channel to get coordinates for.
    :type datetime: :class:`~obspy.core.utcdatetime.UTCDateTime`, optional
    :param datetime: Time to get coordinates for.
    :rtype: dict
    :return: Dictionary containing coordinates (latitude, longitude,
        elevation)
    """
    network, station, location, channel = seed_id.split(".")
    coordinates = []
    if xml_network.code != network:
        pass
    elif xml_network.start_date and xml_network.start_date > datetime:
        pass
    elif xml_network.end_date and xml_network.end_date < datetime:
        pass
    else:
        for sta in xml_network.stations:
            # skip wrong station
            if sta.code != station:
                continue
            # check datetime only if given
            if datetime:
                # skip if start date before given datetime
                if sta.start_date and sta.start_date > datetime:
                    continue
                # skip if end date before given datetime
                if sta.end_date and sta.end_date < datetime:
                    continue
            for cha in sta.channels:
                # skip wrong channel
                if cha.code != channel:
                    continue
                # skip wrong location
                if cha.location_code != location:
                    continue
                # check datetime only if given
                if datetime:
                    # skip if start date before given datetime
                    if cha.start_date and cha.start_date > datetime:
                        continue
                    # skip if end date before given datetime
                    if cha.end_date and cha.end_date < datetime:
                        continue
                # prepare coordinates
                data = {}
                # if channel latitude or longitude is not given use station
                data['latitude'] = cha.latitude or sta.latitude
                data['longitude'] = cha.longitude or sta.longitude
                data['elevation'] = cha.elevation
                data['local_depth'] = cha.depth
                coordinates.append(data)
    if len(coordinates) > 1:
        msg = "Found more than one matching coordinates. Returning first."
        warnings.warn(msg)
    elif len(coordinates) < 1:
        msg = "No matching coordinates found."
        raise Exception(msg)
    return coordinates[0]

# ##################### plot_xml_plotallstages ##########################


def plot_xml_plotallstages(xml_response, t_samp, nyquist, nfft, min_freq,
                           output, start_stage, end_stage, cha_name):
    """
    plot all the stages in a StationXML file.
    This is controlled by start_stage and end_stage
    :param xml_response:
    :param t_samp:
    :param nyquist:
    :param nfft:
    :param min_freq:
    :param output:
    :param start_stage:
    :param end_stage:
    :param cha_name:
    :return:
    """

    if not os.path.isdir('./stationxml_plots'):
        print '[INFO] creating stationxml_plots directory...',
        os.mkdir('./stationxml_plots')
        print 'DONE'
    plt.close()
    plt.ion()
    plt.figure(figsize=(20, 10))
    plt.suptitle(cha_name, size=24, weight='bold')
    for i in range(start_stage, end_stage+1):
        try:
            cpx_response, freq = xml_response.get_evalresp_response(
                t_samp=t_samp, nfft=nfft, output=output,
                start_stage=i, end_stage=i)
        except Exception, e:
            print 'WARNING: %s' % e
            continue

        try:
            inp = xml_response.response_stages[i-1].input_units
        except Exception, e:
            print 'WARNING: %s' % e
            inp = ''
        try:
            out = xml_response.response_stages[i-1].output_units
        except Exception, e:
            print 'WARNING: %s' % e
            out = ''

        phase_resp = np.angle(cpx_response)

        ax = plt.subplot(2, 1, 1)
        ax.loglog(freq, abs(cpx_response), lw=3,
                  label='%s (%s->%s)' % (i, inp, out))
        ax.axvline(nyquist, ls="--", color='blue', lw=3)
        plt.ylabel('Amplitude', size=24, weight='bold')
        plt.xticks(size=18, weight='bold')
        plt.yticks(size=18, weight='bold')
        plt.xlim(xmin=min_freq, xmax=nyquist+5)
        box = ax.get_position()
        ax.set_position([box.x0, box.y0, box.width * 0.95, box.height])
        ax.legend(loc='center left', prop={'size': 18, 'weight': 'bold'},
                  bbox_to_anchor=(1, 0.5))
        ax.grid()

        ax = plt.subplot(2, 1, 2)
        ax.semilogx(freq, phase_resp, lw=3,
                    label='%s (%s->%s)' % (i, inp, out))
        ax.axvline(nyquist, ls="--", color='blue', lw=3)
        plt.xlabel('Frequency [Hz]', size=24, weight='bold')
        plt.ylabel('Phase [rad]', size=24, weight='bold')
        plt.xticks(size=18, weight='bold')
        plt.yticks(size=18, weight='bold')
        plt.xlim(xmin=min_freq, xmax=nyquist+5)
        box = ax.get_position()
        ax.set_position([box.x0, box.y0, box.width * 0.95, box.height])
        ax.legend(loc='center left', prop={'size': 18, 'weight': 'bold'},
                  bbox_to_anchor=(1, 0.5))
        ax.grid()
    plt.savefig(os.path.join('stationxml_plots',
                             '%s_stages.png' % cha_name))

# ##################### convert_xml_paz ######################################


def convert_xml_paz(xml_response, output, cha_name, cha_date):
    """
    convert Stationxml file into PAZ dictionary
    :param xml_response:
    :param output:
    :param cha_name:
    :param cha_date:
    :return:
    """
    gain_arr = []
    normalization_factor = []
    poles = []
    zeros = []

    decimation_delay = []
    decimation_correction = []

    for resp_stage in xml_response.response_stages:
        gain_arr.append(resp_stage.stage_gain)
        try:
            poles.append(resp_stage.poles)
            zeros.append(resp_stage.zeros)
            normalization_factor.append(resp_stage.normalization_factor)
        except Exception as e:
            pass
        if resp_stage.decimation_delay:
            decimation_delay.append(resp_stage.decimation_delay)
        if resp_stage.decimation_correction:
            decimation_correction.append(resp_stage.decimation_correction)

    if len(poles) > 1:
        print 'WARNING: More than one group of poles was found: %s' % poles
    if len(zeros) > 1:
        print 'WARNING: More than one group of zeros was found: %s' % zeros

    normalization_factor = normalization_factor[0]
    poles = poles[0]
    zeros = zeros[0]

    pz_type = xml_response.response_stages[0].pz_transfer_function_type
    if 'hertz' in pz_type.lower():
        poles = np.array(poles)*2*np.pi
        zeros = np.array(zeros)*2*np.pi
        normalization_factor = \
            normalization_factor*np.power(2*np.pi, len(poles) - len(zeros))
    elif 'radian' in pz_type.lower():
        poles = np.array(poles)
        zeros = np.array(zeros)
    else:
        sys.exit('pz_type: %s' % pz_type)

    paz = {'poles': poles}

    input_units = xml_response.response_stages[0].input_units
    if not input_units.lower() in ['m', 'm/s', 'm/s**2']:
        print('ERROR: input unit is not defined: %s\nContact the developer'
              % input_units)
        error_fio = open(os.path.join('./stationxml_plots',
                                      'error_format'), 'a')
        error_fio.writelines('%s\t\t%s\t\t%s\n' % (cha_name, cha_date,
                                                   input_units))
        return False
    if input_units.lower() == 'm/s':
        if output.lower() == 'disp':
            zeros[0].append(0.0j)
    if input_units.lower() == 'm/s**2':
        if output.lower() == 'disp':
            zeros[0].append(0.0j)
            zeros[0].append(0.0j)
        if output.lower() == 'vel':
            zeros[0].append(0.0j)
    if output.lower() == 'acc':
        sys.exit('%s output has not implemented!' % output)
    paz['zeros'] = zeros
    paz['gain'] = normalization_factor
    paz['sensitivity'] = np.prod(np.array(gain_arr))
    print 'Final PAZ:'
    print paz
    return paz, decimation_delay, decimation_correction

# ##################### seismicity ######################################


def seismicity(input_dics, events):
    """
    Create seismicity map
    :param input_dics:
    :param events:
    :return:
    """
    print '\n##############'
    print 'Seismicity map'
    print '##############\n'

    if input_dics['evlatmin'] is None:
        input_dics['evlatmin'] = -90
        input_dics['evlatmax'] = +90
        input_dics['evlonmin'] = -180
        input_dics['evlonmax'] = +180
        map_proj = 'cyl'
    else:
        map_proj = 'cyl'

    # Set-up the map
    m = Basemap(projection=map_proj,
                llcrnrlat=input_dics['evlatmin'],
                urcrnrlat=input_dics['evlatmax'],
                llcrnrlon=input_dics['evlonmin'],
                urcrnrlon=input_dics['evlonmax'],
                lon_0=0,
                resolution='l')
    parallels = np.arange(-90, 90, 30.)
    m.drawparallels(parallels, labels=[1, 1, 1, 1], fontsize=24)
    meridians = np.arange(-180., 180., 60.)
    m.drawmeridians(meridians, labels=[1, 1, 1, 1], fontsize=24)

    raw_input_resp = raw_input('Choose your map style:\n'
                               '1. Bluemarble (PIL should be installed)\n'
                               '2. Etopo (PIL should be installed)\n'
                               '3. Shaderelief (PIL should be installed)\n'
                               '4. Simple\n')
    if raw_input_resp == '1':
        m.bluemarble(scale=0.5)
    elif raw_input_resp == '2':
        m.etopo(scale=0.5)
    elif raw_input_resp == '3':
        m.shadedrelief(scale=0.1)
    else:
        m.fillcontinents()

    # Defining Labels:
    x_ev, y_ev = m(-360, 0)
    m.scatter(x_ev, y_ev, 20, color='red', marker="o",
              edgecolor="black", zorder=10, label='0-70km')
    m.scatter(x_ev, y_ev, 20, color='green', marker="o",
              edgecolor="black", zorder=10, label='70-300km')
    m.scatter(x_ev, y_ev, 20, color='blue', marker="o",
              edgecolor="black", zorder=10, label='300< km')

    m.scatter(x_ev, y_ev, 10, color='white', marker="o",
              edgecolor="black", zorder=10, label='<=4.0')
    m.scatter(x_ev, y_ev, 40, color='white', marker="o",
              edgecolor="black", zorder=10, label='4.0-5.0')
    m.scatter(x_ev, y_ev, 70, color='white', marker="o",
              edgecolor="black", zorder=10, label='5.0-6.0')
    m.scatter(x_ev, y_ev, 100, color='white', marker="o",
              edgecolor="black", zorder=10, label='6.0<')

    ev_dp_all = []
    ev_mag_all = []
    ev_info_ar = np.array([])
    plot_focal_mechanism = False
    for ev in events:
        x_ev, y_ev = m(float(ev['longitude']), float(ev['latitude']))
        ev_dp_all.append(abs(float(ev['depth'])))
        ev_mag_all.append(abs(float(ev['magnitude'])))
        if abs(float(ev['depth'])) <= 70.0:
            color = 'red'
        elif 70.0 < abs(float(ev['depth'])) <= 300.0:
            color = 'green'
        elif 300.0 < abs(float(ev['depth'])) <= 1000.0:
            color = 'blue'

        if float(ev['magnitude']) <= 4.0:
            size = 10
        elif 4.0 < float(ev['magnitude']) <= 5.0:
            size = 40
        elif 5.0 < float(ev['magnitude']) <= 6.0:
            size = 70
        elif 6.0 < float(ev['magnitude']):
            size = 100

        if ev['focal_mechanism']:
            plot_focal_mechanism = True
            f1 = ev['focal_mechanism'][0]
            f2 = ev['focal_mechanism'][1]
            f3 = ev['focal_mechanism'][2]
            f4 = ev['focal_mechanism'][3]
            f5 = ev['focal_mechanism'][4]
            f6 = ev['focal_mechanism'][5]
        else:
            f1 = False
            f2 = False
            f3 = False
            f4 = False
            f5 = False
            f6 = False
        if np.size(ev_info_ar) < 1:
            ev_info_ar = np.append(ev_info_ar,
                                   [float(ev['depth']), float(x_ev),
                                    float(y_ev), size, color,
                                    f1, f2, f3, f4, f5, f6])
        else:
            ev_info_ar = np.vstack((ev_info_ar,
                                    [float(ev['depth']), float(x_ev),
                                     float(y_ev), size, color,
                                     f1, f2, f3, f4, f5, f6]))

    if np.shape(ev_info_ar)[0] == np.size(ev_info_ar):
        ev_info_ar = np.reshape(ev_info_ar, (1, 11))
    else:
        ev_info_ar = sorted(ev_info_ar,
                            key=lambda ev_info_iter: float(ev_info_iter[0]))

    for ev in ev_info_ar:
        m.scatter(float(ev[1]), float(ev[2]), float(ev[3]),
                  color=ev[4], marker="o", edgecolor=None, zorder=10)

    plt.legend(bbox_to_anchor=(1.01, 1), loc=2, borderaxespad=0.)

    if plot_focal_mechanism:
        plt.figure()
        # Set-up the map
        m = Basemap(projection=map_proj,
                    llcrnrlat=input_dics['evlatmin'],
                    urcrnrlat=input_dics['evlatmax'],
                    llcrnrlon=input_dics['evlonmin'],
                    urcrnrlon=input_dics['evlonmax'],
                    lon_0=0,
                    resolution='l')
        parallels = np.arange(-90, 90, 30.)
        m.drawparallels(parallels, labels=[1, 1, 1, 1], fontsize=24)
        meridians = np.arange(-180., 180., 60.)
        m.drawmeridians(meridians, labels=[1, 1, 1, 1], fontsize=24)
        if raw_input_resp == '1':
            m.bluemarble(scale=0.5)
        elif raw_input_resp == '2':
            m.etopo(scale=0.5)
        elif raw_input_resp == '3':
            m.shadedrelief(scale=0.1)
        else:
            m.fillcontinents()
        for evfoc in ev_info_ar:
            try:
                ax = plt.gca()
                focmec = (float(evfoc[5]), float(evfoc[6]), float(evfoc[7]),
                          float(evfoc[8]), float(evfoc[9]), float(evfoc[10]))
                b = Beach(focmec, xy=(float(evfoc[1]), float(evfoc[2])),
                          facecolor=evfoc[4], width=float(evfoc[3])/100.,
                          linewidth=1, alpha=0.85)
                b.set_zorder(10)
                ax.add_collection(b)
            except Exception, e:
                print 'EXCEPTION: %s' % e
                print 'Focal Mechanism:'
                print focmec
                print '----------------'

    if not len(ev_dp_all) <= 1:
        plt.figure()
        plt.hist(ev_dp_all, input_dics['depth_bins_seismicity'],
                 facecolor='green', alpha=0.75, log=True,
                 histtype='stepfilled')
        plt.xlabel('Depth', size=24, weight='bold')
        plt.ylabel('#Events (log)', size=24, weight='bold')
        plt.yscale('log')
        plt.xticks(size=18, weight='bold')
        plt.yticks(size=18, weight='bold')
        plt.tight_layout()

        plt.figure()
        plt.hist(ev_mag_all,
                 bins=np.linspace(int(float(input_dics['min_mag'])),
                                  int(float(input_dics['max_mag'])),
                                  (int(float(input_dics['max_mag'])) -
                                  int(float(input_dics['min_mag'])))*2+1),
                 facecolor='green', alpha=0.75, log=True,
                 histtype='stepfilled')
        plt.xlabel('Magnitude', size=24, weight='bold')
        plt.ylabel('#Events (log)', size=24, weight='bold')
        plt.yscale('log')
        plt.xticks(size=18, weight='bold')
        plt.yticks(size=18, weight='bold')
        plt.tight_layout()

    plt.show()

# ##################### plot_dt #########################################


def plot_dt(input_dics, address_events):
    """
    Plot stored Data(MB) as a function of Time(Sec)
    :param input_dics:
    :param address_events:
    :return:
    """

    single_succ = None
    parallel_succ = None
    for i in range(len(address_events)):
        for client_time in ['time_fdsn', 'time_arc']:
            print 'Event address: %s' % address_events[i]
            if os.path.isfile(os.path.join(address_events[i],
                                           'info',
                                           client_time)):
                plt.clf()
                dt_open = open(os.path.join(address_events[i],
                                            'info',
                                            client_time))
                dt_read = dt_open.readlines()
                for j in range(len(dt_read)):
                    dt_read[j] = dt_read[j].split(',')

                time_single = 0
                succ = 0
                fail = 0
                MB_all = []
                time_all = []

                for k in range(len(dt_read)):
                    time_single += eval(dt_read[k][4]) + \
                                   eval(dt_read[k][5])/1.e6
                    time_all.append(time_single)

                    MB_single = eval(dt_read[k][6])
                    MB_all.append(MB_single)

                    if dt_read[k][7] == '+':
                        single_succ = plt.scatter(time_single,
                                                  MB_single,
                                                  s=1, c='b',
                                                  edgecolors='b', marker='o',
                                                  label='Serial (successful)')
                        succ += 1
                    elif dt_read[k][7] == '-':
                        # single_fail = plt.scatter(time_single,
                        #                           MB_single,
                        #                           s=1, c='r',
                        #                           edgecolors='r', marker='o',
                        #                           label='Serial (failed)')
                        fail += 1

                if input_dics['req_parallel'] == 'Y':
                    rep_par_open = open(os.path.join(address_events[i],
                                                     'info',
                                                     'report_parallel'))
                    rep_par_read = rep_par_open.readlines()
                    time_parallel = \
                        eval(rep_par_read[4].split(',')[0]) + \
                        eval(rep_par_read[4].split(',')[1])/1.e6
                    MB_parallel = eval(rep_par_read[4].split(',')[2])
                    # trans_rate_parallel = MB_parallel/time_parallel*60
                    parallel_succ = plt.scatter(time_parallel,
                                                MB_parallel,
                                                s=30, c='r',
                                                edgecolors='r', marker='o',
                                                label='Parallel')

                time_array = np.array(time_all)
                MB_array = np.array(MB_all)

                poly = np.poly1d(np.polyfit(time_array, MB_array, 1))
                plt.plot(time_array, poly(time_array), 'k--')

                trans_rate = (poly(time_array[-1]) - poly(time_array[0])) / \
                             (time_array[-1]-time_array[0])

                plt.xlabel('Time (sec)', size='large', weight='bold')
                plt.ylabel('Stored Data (MB)', size='large', weight='bold')
                plt.xticks(size='large', weight='bold')
                plt.yticks(size='large', weight='bold')
                plt_title = \
                    '%s\nAll: %s--Succ: %s (%s%%)-Fail: %s (%s%%)--%sMB/sec' \
                    % (client_time.split('_')[1].upper(), (succ + fail), succ,
                       round(float(succ)/(succ + fail)*100., 1), fail,
                       round(float(fail)/(succ + fail)*100., 1),
                       round(trans_rate, 2))
                plt.title(plt_title, size='x-large')

                if input_dics['req_parallel'] == 'Y':
                    plt.legend([single_succ, parallel_succ],
                               ['Serial', 'Parallel'], loc=4)

                plt.savefig(
                    os.path.join('data-time_%s_%s.%s'
                                 % (client_time.split('_')[1],
                                    os.path.basename(address_events[i]),
                                    input_dics['plot_format'])))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# -------------------------------------------------------------------
#   Filename:  input_handler.py
#   Purpose:   reading and generating input_dics
#   Author:    Kasra Hosseini
#   Email:     hosseini@geophysik.uni-muenchen.de
#   License:   GPLv3
# -------------------------------------------------------------------

# -----------------------------------------------------------------------
# ----------------Import required Modules (Python and Obspy)-------------
# -----------------------------------------------------------------------

# Required Python and Obspy modules will be imported in this part.
from obspy.core import UTCDateTime
from optparse import OptionParser, OptionGroup
import os
import shutil
import sys
import time

#  ##################### descrip_generator ###################################


def descrip_generator():
    print "*********************************"
    print "Check all the BASIC dependencies:"
    try:
        from obspy import __version__ as obs_ver
    except Exception as error:
        print '---------------------------------------------------'
        print 'Have you properly installed ObsPy on your computer?'
        print 'Error: %s' % error
        print '---------------------------------------------------'
        sys.exit(2)

    descrip = ['obspy ver: ' + obs_ver]

    import numpy as np
    descrip.append('numpy ver: ' + np.__version__)
    import scipy
    descrip.append('scipy ver: ' + scipy.__version__)

    # Import plotting modules:
    try:
        from matplotlib import __version__ as mat_ver
        import matplotlib.pyplot as plt
        descrip.append('matplotlib ver: ' + mat_ver)
    except Exception as error:
        descrip.append('matplotlib: not installed\n\nerror:\n%s\n' % error)

    try:
        from mpl_toolkits.basemap import __version__ as base_ver
        from mpl_toolkits.basemap import Basemap
        descrip.append('Basemap ver: ' + base_ver)
    except Exception as error:
        descrip.append('Basemap: not installed\n\nerror:\n%s\n'
                       'You could not use all the plot options' % error)
    return descrip

#  ##################### command_parse ###################################


def command_parse():
    """
    Parsing command-line options.
    :return:
    """
    # create command line option parser
    parser = OptionParser("%prog [options]")
    # configure command line options
    # action=".." tells OptionsParser what to save:
    # store_true saves bool TRUE,
    # store_false saves bool FALSE, store saves string; into the variable
    # given with dest="var"
    # * you need to provide every possible option here.

    helpmsg = "Show the description of all option groups"
    parser.add_option("--options", action="store_true", default=False,
                      dest="options", help=helpmsg)

    helpmsg = "Show the options inside specified option group. " \
              "Syntax: --option 1, " \
              "the numbers can be seen from --options flag."
    parser.add_option("--list_option", action="store", default=False,
                      dest="list_option", help=helpmsg)

    # --------------- Check installation and obspyDMT version
    group_check = OptionGroup(parser,
                              "1. Check installation and obspyDMT version")
    helpmsg = "run a quick tour!"
    group_check.add_option("--tour", action="store_true",
                           dest="tour", help=helpmsg)

    helpmsg = "check all the dependencies and their installed versions on " \
              "the local machine and exit!"
    group_check.add_option("--check", action="store_true",
                           dest="check", help=helpmsg)

    helpmsg = "show the obspyDMT version and exit!"
    group_check.add_option("--version", action="store_true",
                           dest="version", help=helpmsg)
    parser.add_option_group(group_check)

    # --------------- Path specification
    group_path = OptionGroup(parser, "2. Path specification")
    helpmsg = "the path where obspyDMT will store the data " \
              "[Default: './obspyDMT-data']"
    group_path.add_option("--datapath", action="store",
                          dest="datapath", help=helpmsg)

    helpmsg = "if the datapath is found deleting it before running obspyDMT."
    group_path.add_option("--reset", action="store_true",
                          dest="reset", help=helpmsg)

    parser.add_option_group(group_path)

    # --------------- General options for requests
    group_req = OptionGroup(parser, "3. General options for requests")
    helpmsg = "continuous request (please refer to the tutorial)."
    group_req.add_option("--continuous", action="store_true",
                         dest="get_continuous", help=helpmsg)

    helpmsg = "event-based request " \
              "(please refer to the tutorial). [Default: 'Y']"
    group_req.add_option("--get_events", action="store",
                         dest="get_events", help=helpmsg)

    helpmsg = "retrieve waveform(s). [Default: 'Y']"
    group_req.add_option("--waveform", action="store",
                         dest="waveform", help=helpmsg)

    helpmsg = "send request (waveform/response) to FDSN. [Default: 'Y']"
    group_req.add_option("--FDSN", action="store",
                         dest="FDSN", help=helpmsg)

    helpmsg = "send request (waveform/response) to ArcLink. [Default: 'N']"
    group_req.add_option("--arc", action="store",
                         dest="ArcLink", help=helpmsg)

    helpmsg = "retrieve the response file. [Default: 'Y']"
    group_req.add_option("--response", action="store",
                         dest="response", help=helpmsg)

    helpmsg = "retrieve the PAZ."
    group_req.add_option("--paz", action="store_true",
                         dest="paz", help=helpmsg)

    helpmsg = "parallel waveform/response/paz request"
    group_req.add_option("--req_parallel", action="store_true",
                         dest="req_parallel", help=helpmsg)

    helpmsg = "number of processors to be used in --req_parallel. [Default: 4]"
    group_req.add_option("--req_np", action="store",
                         dest="req_np", help=helpmsg)

    helpmsg = "use a station list instead of checking the availability. " \
              "[Default: False]"
    group_req.add_option("--list_stas", action="store",
                         dest="list_stas", help=helpmsg)

    helpmsg = "retrieve synthetic waveforms calculated by normal mode " \
              "summation code. (ShakeMovie project)"
    group_req.add_option("--normal_mode_syn", action="store_true",
                         dest="normal_mode_syn", help=helpmsg)

    helpmsg = "retrieve synthetic waveforms of SPECFEM3D."
    group_req.add_option("--specfem3D", action="store_true",
                         dest="specfem3D", help=helpmsg)

    helpmsg = "test the program for the desired number of requests, " \
              "eg: '--test 10' will test the program for 10 " \
              "requests. [Default: 'N']"
    group_req.add_option("--test", action="store",
                         dest="test", help=helpmsg)
    parser.add_option_group(group_req)

    # --------------- Continuous request
    group_cont = OptionGroup(parser, "4. Continuous request")
    helpmsg = "time interval for dividing the continuous request. " \
              "[Default: 86400 sec (1 day)]"
    group_cont.add_option("--interval", action="store",
                          dest="interval", help=helpmsg)
    helpmsg = "preset defined for EACH continuous request, i.e. time before " \
              "EACH interval (refer to '--interval' option) " \
              "in continuous request. [Default: 0]"
    group_cont.add_option("--preset_cont", action="store",
                          dest="preset_cont", help=helpmsg)

    helpmsg = "offset defined for EACH continuous request, i.e. time after " \
              "EACH interval (refer to '--interval' option) " \
              "in continuous request. [Default: 0]"
    group_cont.add_option("--offset_cont", action="store",
                          dest="offset_cont", help=helpmsg)
    parser.add_option_group(group_cont)

    # --------------- Events
    group_ev = OptionGroup(parser, "5. Events")
    helpmsg = "just retrieve the event information and " \
              "create an event archive."
    group_ev.add_option("--event_info", action="store_true",
                        dest="event_info", help=helpmsg)

    helpmsg = "read in an existing event catalog and proceed. " \
              "Currently supported data formats: " \
              "'QUAKEML', 'MCHEDR' e.g.: --read_catalog 'path/to/file'"
    group_ev.add_option("--read_catalog", action="store",
                        dest="read_catalog", help=helpmsg)

    helpmsg = "user can interactively select events of retrieved event " \
              "catalog"
    group_ev.add_option("--user_select_event", action="store_true",
                        dest="user_select_event", help=helpmsg)

    helpmsg = "start time, syntax: Y-M-D-H-M-S " \
              "(eg: '2010-01-01-00-00-00') or just " \
              "Y-M-D [Default: 10 days ago]"
    group_ev.add_option("--min_date", action="store",
                        dest="min_date", help=helpmsg)

    helpmsg = "end time, syntax: Y-M-D-H-M-S " \
              "(eg: '2011-01-01-00-00-00') or just " \
              "Y-M-D [Default: 5 days ago]"
    group_ev.add_option("--max_date", action="store",
                        dest="max_date", help=helpmsg)

    helpmsg = "minimum depth. [Default: -10.0 (above the surface!)]"
    group_ev.add_option("--min_depth", action="store",
                        dest="min_depth", help=helpmsg)

    helpmsg = "maximum depth. [Default: +6000.0]"
    group_ev.add_option("--max_depth", action="store",
                        dest="max_depth", help=helpmsg)

    helpmsg = "minimum magnitude. [Default: 5.5]"
    group_ev.add_option("--min_mag", action="store",
                        dest="min_mag", help=helpmsg)

    helpmsg = "maximum magnitude. [Default: 9.9]"
    group_ev.add_option("--max_mag", action="store",
                        dest="max_mag", help=helpmsg)

    helpmsg = "magnitude type. " \
              "Some common types (there are many) include " \
              "'Ml' (local/Richter magnitude), " \
              "'Ms' (surface magnitude), " \
              "'mb' (body wave magnitude), " \
              "'Mw' (moment magnitude). " \
              "[Default: None]"
    group_ev.add_option("--mag_type", action="store",
                        dest="mag_type", help=helpmsg)

    helpmsg = "search for all the events within the defined rectangle, " \
              "GMT syntax: <lonmin>/<lonmax>/<latmin>/<latmax> " \
              "[Default: -180.0/+180.0/-90.0/+90.0]"
    group_ev.add_option("--event_rect", action="store",
                        dest="event_rect", help=helpmsg)

    helpmsg = "search for all the events within the defined circle, " \
              "syntax: <lon>/<lat>/<rmin>/<rmax>. " \
              "May not be used together with rectangular bounding box " \
              "event restrictions (event_rect)."
    group_ev.add_option("--event_circle", action="store",
                        dest="event_circle", help=helpmsg)

    helpmsg = "event webservice (IRIS or NERIES). [Default: 'IRIS']"
    group_ev.add_option("--event_url", action="store",
                        dest="event_url", help=helpmsg)

    helpmsg = "event catalog (GCMT_COMBO, IRIS, ISC, EMSC, GCMT, NEIC PDE). " \
              "[Default: IRIS]"
    group_ev.add_option("--event_catalog", action="store",
                        dest="event_catalog", help=helpmsg)

    helpmsg = "maximum number of events to be requested. [Default: 2500]"
    group_ev.add_option("--max_result", action="store",
                        dest="max_result", help=helpmsg)
    parser.add_option_group(group_ev)

    # --------------- Stations
    group_sta = OptionGroup(parser, "6. Stations")
    helpmsg = "identity code restriction, syntax: " \
              "net.sta.loc.cha (eg: TA.*.*.BHZ to search for " \
              "all BHZ channels in TA network). [Default: *.*.*.*]"
    group_sta.add_option("--identity", action="store",
                         dest="identity", help=helpmsg)

    helpmsg = "network code. [Default: *]"
    group_sta.add_option("--net", action="store",
                         dest="net", help=helpmsg)

    helpmsg = "station code. [Default: *]"
    group_sta.add_option("--sta", action="store",
                         dest="sta", help=helpmsg)

    helpmsg = "location code. [Default: *]"
    group_sta.add_option("--loc", action="store",
                         dest="loc", help=helpmsg)

    helpmsg = "channel code. [Default: *]"
    group_sta.add_option("--cha", action="store",
                         dest="cha", help=helpmsg)

    helpmsg = "search for all the stations within the defined rectangle, " \
              "GMT syntax: <lonmin>/<lonmax>/<latmin>/<latmax>. " \
              "May not be used together with circular bounding box station " \
              "restrictions (station_circle) " \
              "[Default: -180.0/+180.0/-90.0/+90.0]"
    group_sta.add_option("--station_rect", action="store",
                         dest="station_rect", help=helpmsg)

    helpmsg = "search for all the stations within the defined circle, " \
              "syntax: <lon>/<lat>/<rmin>/<rmax>. " \
              "May not be used together with rectangular bounding box " \
              "station restrictions (station_rect). Currently, " \
              "ArcLink does not support this option!"
    group_sta.add_option("--station_circle", action="store",
                         dest="station_circle", help=helpmsg)
    parser.add_option_group(group_sta)

    # --------------- Time window and waveform format
    group_tw = OptionGroup(parser, "7. Time window and waveform format")
    helpmsg = "time parameter in seconds which determines " \
              "how close the time series data (waveform) will be cropped " \
              "before the origin time of the event. Default: 0.0 seconds."
    group_tw.add_option("--preset", action="store",
                        dest="preset", help=helpmsg)

    helpmsg = "time parameter in seconds which determines " \
              "how close the time series data (waveform) will be cropped " \
              "after the origin time of the event. Default: 1800.0 seconds."
    group_tw.add_option("--offset", action="store",
                        dest="offset", help=helpmsg)

    helpmsg = "consider the first phase arrival (P, Pdiff, PKIKP) to use " \
              "as the reference time, i.e. --min_date and --max_date will " \
              "be calculated from the first phase arrival."
    group_tw.add_option("--cut_time_phase", action="store_true",
                        dest="cut_time_phase", help=helpmsg)

    helpmsg = "SAC format for saving the waveforms. " \
              "Station location (stla and stlo), " \
              "station elevation (stel), " \
              "station depth (stdp), " \
              "event location (evla and evlo), " \
              "event depth (evdp) and " \
              "event magnitude (mag) " \
              "will be stored in the SAC headers. [Default: 'Y'] "
    group_tw.add_option("--SAC", action="store",
                        dest="SAC", help=helpmsg)

    helpmsg = "MSEED format for saving the waveforms."
    group_tw.add_option("--mseed", action="store_true",
                        dest="mseed", help=helpmsg)
    parser.add_option_group(group_tw)

    # --------------- FDSN
    group_fdsn = OptionGroup(parser, "8. FDSN")
    helpmsg = "base_url for FDSN requests (waveform/response). " \
              "[Default: 'IRIS']"
    group_fdsn.add_option("--fdsn_base_url", action="store",
                          dest="fdsn_base_url", help=helpmsg)

    helpmsg = "using the FDSN bulkdataselect Web service. " \
              "Since this method returns multiple channels of " \
              "time series data for specified time ranges in one request, " \
              "it speeds up the waveform retrieving approximately by " \
              "a factor of two. [RECOMMENDED]"
    group_fdsn.add_option("--fdsn_bulk", action="store_true",
                          dest="fdsn_bulk", help=helpmsg)

    helpmsg = "username for FDSN requests (waveform/response). [Default: None]"
    group_fdsn.add_option("--fdsn_user", action="store",
                          dest="fdsn_user", help=helpmsg)

    helpmsg = "password for FDSN requests (waveform/response). [Default: None]"
    group_fdsn.add_option("--fdsn_pass", action="store",
                          dest="fdsn_pass", help=helpmsg)

    helpmsg = "generate a data-time file for a FDSN request. " \
              "This file shows the required time for each request and " \
              "the stored data in the folder."
    group_fdsn.add_option("--time_fdsn", action="store_true",
                          dest="time_fdsn", help=helpmsg)
    parser.add_option_group(group_fdsn)

    # --------------- ArcLink
    group_arc = OptionGroup(parser, "9. ArcLink")
    helpmsg = "timeout for sending request (availability) to ArcLink. " \
              "[Default: 40]"
    group_arc.add_option("--arc_avai_timeout", action="store",
                         dest="arc_avai_timeout", help=helpmsg)

    helpmsg = "timeout for sending request (waveform/response) to ArcLink. " \
              "[Default: 2]"
    group_arc.add_option("--arc_wave_timeout", action="store",
                         dest="arc_wave_timeout", help=helpmsg)

    helpmsg = "generate a data-time file for an ArcLink request. " \
              "This file shows the required time for each request " \
              "and the stored data in the folder."
    group_arc.add_option("--time_arc", action="store_true",
                         dest="time_arc", help=helpmsg)
    parser.add_option_group(group_arc)

    # --------------- Instrument correction
    group_ic = OptionGroup(parser, "10. Instrument correction")
    helpmsg = "correct the raw waveforms for DIS (m), VEL (m/s) or " \
              "ACC (m/s^2). [Default: DIS]"
    group_ic.add_option("--corr_unit", action="store",
                        dest="corr_unit", help=helpmsg)

    helpmsg = "apply a bandpass filter to the data trace before " \
              "deconvolution ('None' if you do not need pre_filter), " \
              "syntax: '(f1,f2,f3,f4)' which " \
              "are the four corner frequencies " \
              "of a cosine taper, one between f2 and f3 and tapers to zero " \
              "for f1 < f < f2 and f3 < f < f4. " \
              "[Default: '(0.008, 0.012, 3.0, 4.0)']"
    group_ic.add_option("--pre_filt", action="store",
                        dest="pre_filt", help=helpmsg)

    helpmsg = "water level for spectrum [Default: 600.0]"
    group_ic.add_option("--water_level", action="store",
                        dest="water_level", help=helpmsg)

    helpmsg = "parallel Instrument Correction. "
    group_ic.add_option("--ic_parallel", action="store_true",
                        dest="ic_parallel", help=helpmsg)

    helpmsg = "number of processors to be used in --ic_parallel. [Default: 20]"
    group_ic.add_option("--ic_np", action="store",
                        dest="ic_np", help=helpmsg)

    helpmsg = "apply instrument correction to the specified folder for " \
              "all the waveforms (FDSN and ArcLink), " \
              "syntax: --ic_all address_of_the_target_folder. [Default: 'N']"
    group_ic.add_option("--ic_all", action="store",
                        dest="ic_all", help=helpmsg)

    helpmsg = "apply instrument correction to the specified folder for " \
              "downloaded waveforms from FDSN, " \
              "syntax: --fdsn_ic address_of_the_target_folder. [Default: 'N']"
    group_ic.add_option("--fdsn_ic", action="store",
                        dest="fdsn_ic", help=helpmsg)

    helpmsg = "apply instrument correction to the specified folder for " \
              "downloaded waveforms from ArcLink, " \
              "syntax: --arc_ic address_of_the_target_folder. [Default: 'N']"
    group_ic.add_option("--arc_ic", action="store",
                        dest="arc_ic", help=helpmsg)

    helpmsg = "apply instrument correction automatically " \
              "after downloading the waveforms from FDSN. [Default: 'Y']"
    group_ic.add_option("--fdsn_ic_auto", action="store",
                        dest="fdsn_ic_auto", help=helpmsg)

    helpmsg = "apply instrument correction automatically " \
              "after downloading the waveforms from ArcLink. [Default: 'Y']"
    group_ic.add_option("--arc_ic_auto", action="store",
                        dest="arc_ic_auto", help=helpmsg)

    helpmsg = "do not apply instrument correction automatically. " \
              "This is equivalent to: \"--fdsn_ic_auto N --arc_ic_auto N\""
    group_ic.add_option("--ic_no", action="store_true",
                        dest="ic_no", help=helpmsg)

    helpmsg = "instrument Correction (full response), using obspy modules. " \
              "[Default: 'Y']"
    group_ic.add_option("--ic_obspy_full", action="store",
                        dest="ic_obspy_full", help=helpmsg)

    helpmsg = "instrument Correction (full response), using SAC"
    group_ic.add_option("--ic_sac_full", action="store_true",
                        dest="ic_sac_full", help=helpmsg)

    helpmsg = "instrument Correction (Poles And Zeros), " \
              "using SAC (for FDSN) and obspy (for ArcLink)"
    group_ic.add_option("--ic_paz", action="store_true",
                        dest="ic_paz", help=helpmsg)
    parser.add_option_group(group_ic)

    # --------------- Updating
    group_up = OptionGroup(parser, "11. Updating")
    helpmsg = "update the specified folder for FDSN and ArcLink, " \
              "syntax: --update_all address_of_the_target_folder. " \
              "[Default: 'N']"
    group_up.add_option("--update_all", action="store",
                        dest="update_all", help=helpmsg)

    helpmsg = "update the specified folder for FDSN, " \
              "syntax: --fdsn_update address_of_the_target_folder. " \
              "[Default: 'N']"
    group_up.add_option("--fdsn_update", action="store",
                        dest="fdsn_update", help=helpmsg)

    helpmsg = "update the specified folder for ArcLink, " \
              "syntax: --arc_update address_of_the_target_folder. " \
              "[Default: 'N']"
    group_up.add_option("--arc_update", action="store",
                        dest="arc_update", help=helpmsg)
    parser.add_option_group(group_up)

    # --------------- Merging
    group_merg = OptionGroup(parser, "12. Merging")
    helpmsg = "merge 'raw' or 'corrected' waveforms. [Default: 'raw']"
    group_merg.add_option("--merge_type", action="store",
                          dest="merge_type", help=helpmsg)

    helpmsg = "merge all waveforms (FDSN and ArcLink) in " \
              "the specified folder, " \
              "syntax: --merge_all address_of_the_target_folder. " \
              "[Default: 'N']"
    group_merg.add_option("--merge_all", action="store",
                          dest="merge_all", help=helpmsg)

    helpmsg = "merge the FDSN waveforms in the specified folder, " \
              "syntax: --fdsn_merge address_of_the_target_folder. " \
              "[Default: 'N']"
    group_merg.add_option("--fdsn_merge", action="store",
                          dest="fdsn_merge", help=helpmsg)

    helpmsg = "merge the ArcLink waveforms in the specified folder, " \
              "syntax: --arc_merge address_of_the_target_folder." \
              "[Default: 'N']"
    group_merg.add_option("--arc_merge", action="store",
                          dest="arc_merge", help=helpmsg)

    helpmsg = "merge automatically after downloading the waveforms " \
              "from FDSN. [Default: 'Y']"
    group_merg.add_option("--fdsn_merge_auto", action="store",
                          dest="fdsn_merge_auto", help=helpmsg)

    helpmsg = "merge automatically after downloading the waveforms " \
              "from ArcLink. [Default: 'Y']"
    group_merg.add_option("--arc_merge_auto", action="store",
                          dest="arc_merge_auto", help=helpmsg)

    helpmsg = "do not merge automatically. This is equivalent to: " \
              "\"--fdsn_merge_auto N --arc_merge_auto N\""
    group_merg.add_option("--merge_no", action="store_true",
                          dest="merge_no", help=helpmsg)
    parser.add_option_group(group_merg)

    # --------------- Plotting
    group_plt = OptionGroup(parser, "13. Plotting")
    helpmsg = "create a seismicity map according to " \
              "the event and location specifications."
    group_plt.add_option("--seismicity", action="store_true",
                         dest="seismicity", help=helpmsg)

    helpmsg = "depth bins for plotting the seismicity histrogram. " \
              "[Default: 10]"
    group_plt.add_option("--depth_bins_seismicity", action="store",
                         dest="depth_bins_seismicity", help=helpmsg)

    helpmsg = "specify directory for plotting purposes [Default: 'N']"
    group_plt.add_option("--plot_dir", action="store",
                         dest="plot_dir", help=helpmsg)

    helpmsg = "plot 'raw' or 'corrected' waveforms. [Default: 'raw']"
    group_plt.add_option("--plot_type", action="store",
                         dest="plot_type", help=helpmsg)

    helpmsg = "plot all waveforms (FDSN and ArcLink). [Default: 'Y']"
    group_plt.add_option("--plot_all", action="store",
                         dest="plot_all", help=helpmsg)

    helpmsg = "plot waveforms downloaded from FDSN."
    group_plt.add_option("--plot_fdsn", action="store_true",
                         dest="plot_fdsn", help=helpmsg)

    helpmsg = "plot waveforms downloaded from ArcLink."
    group_plt.add_option("--plot_arc", action="store_true",
                         dest="plot_arc", help=helpmsg)

    helpmsg = "plot \"epicentral distance-time\" for " \
              "all the waveforms found in the specified folder (--plot_dir)."
    group_plt.add_option("--plot_epi", action="store_true",
                         dest="plot_epi", help=helpmsg)

    helpmsg = "plot \"epicentral distance-time\" (refer to --plot_epi') " \
              "for all the waveforms with " \
              "epicentral-distance >= min_epi. [Default: 0.0]"
    group_plt.add_option("--min_epi", action="store",
                         dest="min_epi", help=helpmsg)

    helpmsg = "plot \"epicentral distance-time\" " \
              "(refer to '--plot_epi') for all the waveforms with " \
              "epicentral-distance <= max_epi. [Default: 180.0]"
    group_plt.add_option("--max_epi", action="store",
                         dest="max_epi", help=helpmsg)

    helpmsg = "plot all the events, stations and ray path between them " \
              "found in the specified directory (--plot_dir)."
    group_plt.add_option("--plot_ray_gmt", action="store_true",
                         dest="plot_ray_gmt", help=helpmsg)

    helpmsg = "plot the ray coverage for all the station-event pairs " \
              "found in the specified folder (--plot_dir)."
    group_plt.add_option("--plot_ray", action="store_true",
                         dest="plot_ray", help=helpmsg)

    helpmsg = "plot all the events found " \
              "in the specified directory (--plot_dir)."
    group_plt.add_option("--plot_ev", action="store_true",
                         dest="plot_ev", help=helpmsg)

    helpmsg = "plot Beachballs instead of dots for the event location."
    group_plt.add_option("--plot_focal", action="store_true",
                         dest="plot_focal", help=helpmsg)

    helpmsg = "plot all the stations found " \
              "in the specified directory (--plot_dir)."
    group_plt.add_option("--plot_sta", action="store_true",
                         dest="plot_sta", help=helpmsg)

    helpmsg = "plot \"Data(MB)-Time(Sec)\" for the " \
              "specified directory (--plot_dir) " \
              "-- ATTENTION: \"time_fdsn\" and/or \"time_arc\" should " \
              "exist in the \"info\" folder [refer to \"time_fdsn\" and " \
              "\"time_arc\" options]"
    group_plt.add_option("--plot_dt", action="store_true",
                         dest="plot_dt", help=helpmsg)

    helpmsg = "the path where obspyDMT will store the plots " \
              "[Default: '.' (the same directory as obspyDMT.py)]"
    group_plt.add_option("--plot_save", action="store",
                         dest="plot_save", help=helpmsg)

    helpmsg = "format of the plots saved on the local machine [Default: 'png']"
    group_plt.add_option("--plot_format", action="store",
                         dest="plot_format", help=helpmsg)
    parser.add_option_group(group_plt)

    # --------------- Plotting SationXML
    group_pltxml = OptionGroup(parser, "14. Plotting StationXML")
    helpmsg = "address of a file/directory that contains StationXML files. " \
              "[Default: False]"
    group_pltxml.add_option("--plotxml_dir", action="store",
                            dest="plotxml_dir", help=helpmsg)

    helpmsg = "plot all the stages available in the response file."
    group_pltxml.add_option("--plotxml_allstages", action="store_true",
                            dest="plotxml_allstages", help=helpmsg)

    helpmsg = "plot PAZ of the response file."
    group_pltxml.add_option("--plotxml_paz", action="store_true",
                            dest="plotxml_paz", help=helpmsg)

    helpmsg = "plot only stage 1 and 2 of full response file."
    group_pltxml.add_option("--plotxml_plotstage12", action="store_true",
                            dest="plotxml_plotstage12", help=helpmsg)

    helpmsg = "start stage in response file to be considered for plotting " \
              "the transfer function. [Default: 1]"
    group_pltxml.add_option("--plotxml_start_stage", action="store",
                            dest="plotxml_start_stage", help=helpmsg)

    helpmsg = "final stage in response file to be considered for plotting " \
              "the transfer function. [Default: 100]"
    group_pltxml.add_option("--plotxml_end_stage", action="store",
                            dest="plotxml_end_stage", help=helpmsg)

    helpmsg = "datetime to be used for plotting the transfer function," \
              "syntax: Y-M-D-H-M-S (eg: '2011-01-01-00-00-00') or just " \
              "Y-M-D. If this is not set, the starting date of the " \
              "stationXML will be used instead!"
    group_pltxml.add_option("--plotxml_date", action="store",
                            dest="plotxml_date", help=helpmsg)

    helpmsg = "minimum frequency to be used for plotting the transfer " \
              "function. [Default: 0.01]"
    group_pltxml.add_option("--plotxml_min_freq", action="store",
                            dest="plotxml_min_freq", help=helpmsg)

    helpmsg = "plot all the stations that have been compared in terms of " \
              "instrument response."
    group_pltxml.add_option("--plotxml_map_compare", action="store_true",
                            dest="plotxml_map_compare", help=helpmsg)

    helpmsg = "percentage of the phase transfer function length to be used " \
              "for checking the difference between different methods, " \
              "e.g. 100 will be the whole transfer function, " \
              "80 means consider the transfer function from min_freq until " \
              "20 percent before the Nyquist frequency. [Default: 80]"
    group_pltxml.add_option("--plotxml_percentage", action="store",
                            dest="plotxml_percentage", help=helpmsg)

    helpmsg = "maximum allowable length (in percentage) to differ between " \
              "two different methods of instrument correction. " \
              "This only applies to phase difference. [Default: 10]"
    group_pltxml.add_option("--plotxml_phase_threshold", action="store",
                            dest="plotxml_phase_threshold", help=helpmsg)

    helpmsg = "output of the transfer function: DIS/VEL/ACC. [Default: VEL]"
    group_pltxml.add_option("--plotxml_output", action="store",
                            dest="plotxml_output", help=helpmsg)

    helpmsg = "do not plot the full response file."
    group_pltxml.add_option("--plotxml_no_response", action="store_true",
                            dest="plotxml_no_response", help=helpmsg)
    parser.add_option_group(group_pltxml)

    # --------------- Email and compressing
    group_ec = OptionGroup(parser, "15. Email and compressing")
    helpmsg = "send an email to the specified email-address after " \
              "completing the job, syntax: --email email_address. " \
              "[Default: 'N']"
    group_ec.add_option("--email", action="store",
                        dest="email", help=helpmsg)

    helpmsg = "compress the raw-waveform files after " \
              "applying instrument correction."
    group_ec.add_option("--zip_w", action="store_true",
                        dest="zip_w", help=helpmsg)

    helpmsg = "compress the response files after " \
              "applying instrument correction."
    group_ec.add_option("--zip_r", action="store_true",
                        dest="zip_r", help=helpmsg)
    parser.add_option_group(group_ec)
    # --------------- END

    # parse command line options
    (options, args) = parser.parse_args()

    return options, args, parser

# ##################### read_input_command ##############################


def read_input_command(parser, **kwargs):
    """
    Create input_dics object (dictionary) based on command-line options.
    The default values are as "input_dics" object (below)
    :param parser:
    :param kwargs:
    :return:
    """
    # Defining the default values.
    input_dics = {'datapath': 'obspyDMT-data',
                  'min_date': str(UTCDateTime() - 60 * 60 * 24 * 10 * 1),
                  'max_date': str(UTCDateTime() - 60 * 60 * 24 * 5 * 1),
                  'event_url': 'IRIS',
                  'event_catalog': 'IRIS',
                  'mag_type': None,
                  'min_mag': 5.5, 'max_mag': 9.9,
                  'min_depth': -10.0, 'max_depth': +6000.0,
                  'get_events': 'Y',
                  'interval': 3600*24,
                  'preset_cont': 0,
                  'offset_cont': 0,
                  'req_np': 4,
                  'list_stas': False,
                  'waveform': 'Y', 'response': 'Y',
                  'FDSN': 'Y', 'ArcLink': 'N',
                  'fdsn_base_url': 'IRIS',
                  'fdsn_user': None,
                  'fdsn_pass': None,
                  'arc_avai_timeout': 40,
                  'arc_wave_timeout': 2,
                  'SAC': 'Y',
                  'preset': 0.0, 'offset': 1800.0,
                  'net': '*', 'sta': '*', 'loc': '*', 'cha': '*',
                  'evlatmin': None, 'evlatmax': None,
                  'evlonmin': None, 'evlonmax': None,
                  'evlat': None, 'evlon': None,
                  'evradmin': None, 'evradmax': None,
                  'max_result': 2500,
                  'depth_bins_seismicity': 10,
                  'lat_cba': None, 'lon_cba': None,
                  'mr_cba': None, 'Mr_cba': None,
                  'mlat_rbb': None, 'Mlat_rbb': None,
                  'mlon_rbb': None, 'Mlon_rbb': None,
                  'test': 'N',
                  'fdsn_update': 'N', 'arc_update': 'N', 'update_all': 'N',
                  'email': 'N',
                  'ic_all': 'N',
                  'fdsn_ic': 'N', 'fdsn_ic_auto': 'Y',
                  'arc_ic': 'N', 'arc_ic_auto': 'Y',
                  'ic_np': 4,
                  'ic_obspy_full': 'Y',
                  'pre_filt': '(0.008, 0.012, 3.0, 4.0)',
                  'water_level': 600.0,
                  'corr_unit': 'DIS',
                  'merge_all': 'N',
                  'fdsn_merge': 'N', 'fdsn_merge_auto': 'Y',
                  'merge_type': 'raw',
                  'arc_merge': 'N', 'arc_merge_auto': 'Y',
                  'plot_dir': 'N',
                  'plot_all': 'Y',
                  'plot_type': 'raw',
                  'plot_save': '.', 'plot_format': 'png',
                  'min_epi': 0.0, 'max_epi': 180.0,
                  'plotxml_dir': False,
                  'plotxml_date': False,
                  'plotxml_min_freq': 0.01,
                  'plotxml_output': 'VEL',
                  'plotxml_start_stage': 1,
                  'plotxml_end_stage': 100,
                  'plotxml_percentage': 80,
                  'plotxml_phase_threshold': 10.,
                  }

    # feed input_dics dictionary of defaults into parser object
    parser.set_defaults(**input_dics)

    # parse command line options
    (options, args) = parser.parse_args()
    # command line options can now be accessed via options.varname.

    # Check if keyword arguments have been passed to the main function from
    # another script and parse here:
    if kwargs:
        # assigning kwargs to entries of OptionParser object
        for arg in kwargs:
            exec "options.%s = kwargs[arg]" % arg

    # printing the description of all option groups
    if options.options:
        print "=============="
        print "option groups:"
        print "=============="
        for grp in parser.option_groups:
            print grp.title
        print "\n\n==============================================="
        print "To check the available options in each group:"
        print "python obspyDMT.py --list_option <group_number>"
        print "==============================================="
        sys.exit()

    # printing the available options in each option group
    if options.list_option:
        if int(options.list_option) > len(parser.option_groups):
            sys.exit('Specified option group: %s does not exist'
                     % options.list_option)
        print parser.option_groups[int(options.list_option)-1].title
        for opt_grp in \
                parser.option_groups[int(options.list_option)-1].option_list:
            print "{0:20s}\t{1:20s}\t{2:20s}\t\t{3:20s}".\
                format(opt_grp.get_opt_string(), opt_grp.dest,
                       opt_grp.type, opt_grp.help)
        sys.exit()

    if options.version:
        print '\n\t\t' + '*********************************'
        print '\t\t' + '*        obspyDMT version:      *'
        print '\t\t' + '*' + '\t\t' + '0.9.9' + '\t\t' + '*'
        print '\t\t' + '*********************************'
        print '\n'
        sys.exit(2)

    # Check whether it is possible to import all required modules
    if options.check:
        descrip = descrip_generator()
        for i in range(len(descrip)):
            print descrip[i]
        print "*********************************\n"
        sys.exit(2)

    if options.tour:
        print '\n########################################'
        print 'obspyDMT Quick Tour will start in 2 sec!'
        print '########################################\n'
        time.sleep(2)
        options.datapath = './dmt-tour-data'
        options.min_date = '2011-03-10'
        options.max_date = '2011-03-12'
        options.min_mag = '8.9'
        options.identity = 'TA.1*.*.BHZ'
        options.event_url = 'IRIS'
        options.event_catalog = 'IRIS'
        options.req_parallel = True
        options.ArcLink = 'N'

    # ############Parse paths and make sure that they are all absolute path
    for paths in ['datapath', 'fdsn_update', 'arc_update', 'update_all',
                  'fdsn_ic', 'arc_ic', 'ic_all', 'fdsn_merge', 'arc_merge',
                  'merge_all', 'plot_dir', 'plot_save', 'plotxml_dir']:
        optatr_path = getattr(options, paths)
        if optatr_path:
            if optatr_path != 'N' and not os.path.isabs(optatr_path):
                setattr(options, paths, os.path.join(os.getcwd(),
                                                     getattr(options, paths)))
    # ############END Parse paths

    # extract min. and max. longitude and latitude for event
    # if the user has given the coordinates with -r (GMT syntax)
    if options.event_rect:
        try:
            options.event_rect = options.event_rect.split('/')
            if len(options.event_rect) != 4:
                print "Erroneous rectangle given."
                sys.exit(2)
            options.evlonmin = float(options.event_rect[0])
            options.evlonmax = float(options.event_rect[1])
            options.evlatmin = float(options.event_rect[2])
            options.evlatmax = float(options.event_rect[3])
        except Exception, e:
            print "Erroneous rectangle given: %s" % e
            sys.exit(2)

    # circular event restriction option parsing
    if options.event_circle:
        try:
            options.event_circle = options.event_circle.split('/')
            if len(options.event_circle) != 4:
                print "Erroneous circle given."
                sys.exit(2)
            options.evlon = float(options.event_circle[0])
            options.evlat = float(options.event_circle[1])
            options.evradmin = float(options.event_circle[2])
            options.evradmax = float(options.event_circle[3])
        except Exception, e:
            print "Erroneous circle given: %s" % e
            sys.exit(2)

    # extract min. and max. longitude and latitude for station
    # if the user has given the coordinates with -g (GMT syntax)
    if options.station_rect:
        try:
            options.station_rect = options.station_rect.split('/')
            if len(options.station_rect) != 4:
                print "Erroneous rectangle given."
                sys.exit(2)
            options.mlon_rbb = float(options.station_rect[0])
            options.Mlon_rbb = float(options.station_rect[1])
            options.mlat_rbb = float(options.station_rect[2])
            options.Mlat_rbb = float(options.station_rect[3])
        except Exception, e:
            print "Erroneous rectangle given: %s" % e
            sys.exit(2)

    # circular station restriction option parsing
    if options.station_circle:
        try:
            options.station_circle = options.station_circle.split('/')
            if len(options.station_circle) != 4:
                print "Erroneous circle given."
                sys.exit(2)
            options.lon_cba = float(options.station_circle[0])
            options.lat_cba = float(options.station_circle[1])
            options.mr_cba = float(options.station_circle[2])
            options.Mr_cba = float(options.station_circle[3])
        except Exception, e:
            print "Erroneous circle given: %s" % e
            sys.exit(2)

    # delete data path if -R or --reset args are given at cmdline
    if options.reset:
        # try-except so we don't get an exception if path doesnt exist
        try:
            shutil.rmtree(options.datapath)
            print '\n-------------------------------'
            print 'Delete the following directory:'
            print str(options.datapath)
            print 'obspyDMT is going to re-create it...'
            print '-------------------------------\n\n'
        except Exception, e:
            print "Warning: can not remove the following directory: %s" % e
            pass

    # Extract network, station, location, channel if the user has given an
    # identity code (-i xx.xx.xx.xx)
    if options.identity:
        try:
            options.net, options.sta, options.loc, options.cha = \
                options.identity.split('.')
        except Exception, e:
            print "Erroneous identity code given: %s" % e
            sys.exit(2)

    input_dics['plotxml_dir'] = options.plotxml_dir
    if options.plotxml_date:
        input_dics['plotxml_date'] = UTCDateTime(options.plotxml_date)
    else:
        input_dics['plotxml_date'] = options.plotxml_date
    input_dics['plotxml_min_freq'] = float(options.plotxml_min_freq)
    input_dics['plotxml_output'] = options.plotxml_output
    input_dics['plotxml_start_stage'] = int(options.plotxml_start_stage)
    input_dics['plotxml_end_stage'] = int(options.plotxml_end_stage)
    input_dics['plotxml_percentage'] = float(options.plotxml_percentage)
    input_dics['plotxml_phase_threshold'] = \
        float(options.plotxml_phase_threshold)
    if options.plotxml_no_response:
        input_dics['plotxml_response'] = False
    else:
        input_dics['plotxml_response'] = True
    if options.plotxml_plotstage12:
        input_dics['plotxml_plotstage12'] = True
    else:
        input_dics['plotxml_plotstage12'] = False
    if options.plotxml_paz:
        input_dics['plotxml_paz'] = True
    else:
        input_dics['plotxml_paz'] = False
    if options.plotxml_allstages:
        input_dics['plotxml_allstages'] = True
    else:
        input_dics['plotxml_allstages'] = False
    if options.plotxml_map_compare:
        input_dics['plotxml_map_compare'] = True
    else:
        input_dics['plotxml_map_compare'] = False
    input_dics['datapath'] = options.datapath
    if options.cut_time_phase:
        input_dics['cut_time_phase'] = True
    else:
        input_dics['cut_time_phase'] = False
    input_dics['min_date'] = str(UTCDateTime(options.min_date))
    input_dics['max_date'] = str(UTCDateTime(options.max_date))
    input_dics['event_url'] = options.event_url.upper()

    if options.event_catalog:
        input_dics['event_catalog'] = options.event_catalog.upper()
    if input_dics['event_catalog'].upper() == 'IRIS':
        input_dics['event_catalog'] = None

    if options.read_catalog:
        input_dics['read_catalog'] = options.read_catalog
    else:
        input_dics['read_catalog'] = 'N'
    input_dics['mag_type'] = options.mag_type
    input_dics['min_mag'] = float(options.min_mag)
    input_dics['max_mag'] = float(options.max_mag)
    input_dics['min_depth'] = float(options.min_depth)
    input_dics['max_depth'] = float(options.max_depth)
    input_dics['evlonmin'] = options.evlonmin
    input_dics['evlonmax'] = options.evlonmax
    input_dics['evlatmin'] = options.evlatmin
    input_dics['evlatmax'] = options.evlatmax
    input_dics['evlat'] = options.evlat
    input_dics['evlon'] = options.evlon
    input_dics['evradmax'] = options.evradmax
    input_dics['evradmin'] = options.evradmin
    input_dics['preset'] = float(options.preset)
    input_dics['offset'] = float(options.offset)
    input_dics['max_result'] = int(options.max_result)
    input_dics['depth_bins_seismicity'] = int(options.depth_bins_seismicity)
    if options.user_select_event:
        input_dics['user_select_event'] = 'Y'
    else:
        input_dics['user_select_event'] = 'N'
    if options.seismicity:
        input_dics['seismicity'] = 'Y'
    else:
        input_dics['seismicity'] = 'N'
    input_dics['get_events'] = options.get_events
    if options.get_continuous:
        input_dics['plot_all_events'] = None
        input_dics['get_events'] = 'N'
        input_dics['get_continuous'] = 'Y'
    else:
        input_dics['get_continuous'] = 'N'
    input_dics['interval'] = float(options.interval)
    input_dics['preset_cont'] = float(options.preset_cont)
    input_dics['offset_cont'] = float(options.offset_cont)
    if options.req_parallel:
        options.req_parallel = 'Y'
    input_dics['req_parallel'] = options.req_parallel
    input_dics['req_np'] = int(options.req_np)
    input_dics['list_stas'] = options.list_stas
    if options.fdsn_bulk:
        options.fdsn_bulk = 'Y'
    input_dics['fdsn_bulk'] = options.fdsn_bulk
    if options.specfem3D:
        options.specfem3D = 'Y'
    input_dics['specfem3D'] = options.specfem3D
    if options.normal_mode_syn:
        options.normal_mode_syn = 'Y'
    input_dics['normal_mode_syn'] = options.normal_mode_syn
    input_dics['waveform'] = options.waveform
    input_dics['response'] = options.response
    if options.paz:
        options.paz = 'Y'
    input_dics['paz'] = options.paz
    input_dics['SAC'] = options.SAC
    if options.mseed:
        input_dics['SAC'] = 'N'
        input_dics['mseed'] = 'Y'
    else:
        input_dics['mseed'] = 'N'
    input_dics['FDSN'] = options.FDSN
    input_dics['fdsn_base_url'] = options.fdsn_base_url
    input_dics['fdsn_user'] = options.fdsn_user
    input_dics['fdsn_pass'] = options.fdsn_pass
    input_dics['ArcLink'] = options.ArcLink

    input_dics['arc_avai_timeout'] = float(options.arc_avai_timeout)
    input_dics['arc_wave_timeout'] = float(options.arc_wave_timeout)

    if options.time_fdsn:
        options.time_fdsn = 'Y'
    input_dics['time_fdsn'] = options.time_fdsn
    if options.time_arc:
        options.time_arc = 'Y'
    input_dics['time_arc'] = options.time_arc
    input_dics['net'] = options.net
    input_dics['sta'] = options.sta
    if options.loc == "''":
        input_dics['loc'] = ''
    elif options.loc == '""':
        input_dics['loc'] = ''
    else:
        input_dics['loc'] = options.loc
    input_dics['cha'] = options.cha
    input_dics['lon_cba'] = options.lon_cba
    input_dics['lat_cba'] = options.lat_cba
    input_dics['mr_cba'] = options.mr_cba
    input_dics['Mr_cba'] = options.Mr_cba
    input_dics['mlon_rbb'] = options.mlon_rbb
    input_dics['Mlon_rbb'] = options.Mlon_rbb
    input_dics['mlat_rbb'] = options.mlat_rbb
    input_dics['Mlat_rbb'] = options.Mlat_rbb
    if options.test != 'N':
        input_dics['test'] = 'Y'
        input_dics['test_num'] = int(options.test)
    input_dics['fdsn_update'] = options.fdsn_update
    input_dics['arc_update'] = options.arc_update
    input_dics['update_all'] = options.update_all
    if input_dics['update_all'] != 'N':
        input_dics['fdsn_update'] = input_dics['update_all']
        input_dics['arc_update'] = input_dics['update_all']
    input_dics['fdsn_ic'] = options.fdsn_ic
    input_dics['fdsn_ic_auto'] = options.fdsn_ic_auto
    input_dics['arc_ic'] = options.arc_ic
    input_dics['arc_ic_auto'] = options.arc_ic_auto
    input_dics['ic_all'] = options.ic_all
    if input_dics['ic_all'] != 'N':
        input_dics['fdsn_ic'] = input_dics['ic_all']
        input_dics['arc_ic'] = input_dics['ic_all']
    if options.ic_parallel:
        options.ic_parallel = 'Y'
    input_dics['ic_parallel'] = options.ic_parallel
    input_dics['ic_np'] = int(options.ic_np)
    input_dics['ic_obspy_full'] = options.ic_obspy_full
    if options.ic_sac_full:
        options.ic_sac_full = 'Y'
    input_dics['ic_sac_full'] = options.ic_sac_full
    if options.ic_paz:
        options.ic_paz = 'Y'
    input_dics['ic_paz'] = options.ic_paz
    if input_dics['ic_sac_full'] == 'Y' or input_dics['ic_paz'] == 'Y':
        input_dics['SAC'] = 'Y'
        input_dics['ic_obspy_full'] = 'N'
    input_dics['corr_unit'] = options.corr_unit
    input_dics['pre_filt'] = options.pre_filt
    input_dics['water_level'] = float(options.water_level)
    if options.zip_w:
        options.zip_w = 'Y'
    input_dics['zip_w'] = options.zip_w
    if options.zip_r:
        options.zip_r = 'Y'
    input_dics['zip_r'] = options.zip_r
    input_dics['fdsn_merge'] = options.fdsn_merge
    input_dics['arc_merge'] = options.arc_merge
    input_dics['merge_all'] = options.merge_all
    if input_dics['merge_all'] != 'N':
        input_dics['fdsn_merge'] = input_dics['merge_all']
        input_dics['arc_merge'] = input_dics['merge_all']
    input_dics['plot_type'] = options.plot_type
    input_dics['plot_all'] = options.plot_all
    if options.plot_fdsn:
        options.plot_fdsn = 'Y'
    input_dics['plot_fdsn'] = options.plot_fdsn
    if options.plot_arc:
        options.plot_arc = 'Y'
    input_dics['plot_arc'] = options.plot_arc
    input_dics['plot_dir'] = options.plot_dir
    if options.plot_ev:
        input_dics['plot_ev'] = True
    else:
        input_dics['plot_ev'] = False
    if options.plot_focal:
        input_dics['plot_focal'] = True
    else:
        input_dics['plot_focal'] = False
    if options.plot_sta:
        input_dics['plot_sta'] = True
    else:
        input_dics['plot_sta'] = False
    if options.plot_ray:
        input_dics['plot_ray'] = True
    else:
        input_dics['plot_ray'] = False
    if options.plot_ray_gmt:
        input_dics['plot_ray_gmt'] = True
    else:
        input_dics['plot_ray_gmt'] = False
    if options.plot_epi:
        input_dics['plot_epi'] = True
    else:
        input_dics['plot_epi'] = False
    if options.plot_dt:
        input_dics['plot_dt'] = True
    else:
        input_dics['plot_dt'] = False
    input_dics['min_epi'] = float(options.min_epi)
    input_dics['max_epi'] = float(options.max_epi)
    input_dics['plot_save'] = options.plot_save
    input_dics['plot_format'] = options.plot_format
    input_dics['email'] = options.email
    if input_dics['email'] != 'N':
        try:
            import smtplib
        except Exception as error:
            print "\n********************************************************"
            print "Unable to import smtplib. Sending email is not possible!"
            print "Error: %s" % error
            print "********************************************************\n"
            sys.exit()

    # --------------Changing relevant options for some specific options
    if input_dics['get_continuous'] == 'N':
        input_dics['fdsn_merge_auto'] = 'N'
        input_dics['arc_merge_auto'] = 'N'
    else:
        input_dics['fdsn_merge_auto'] = options.fdsn_merge_auto
        input_dics['arc_merge_auto'] = options.arc_merge_auto
        input_dics['merge_type'] = options.merge_type

    for opts in ['fdsn_ic', 'arc_ic', 'fdsn_merge', 'arc_merge', 'plot_dir']:
        if input_dics[opts] != 'N':
            input_dics['datapath'] = input_dics[opts]
            input_dics['get_events'] = 'N'
            input_dics['get_continuous'] = 'N'
            input_dics['FDSN'] = 'N'
            input_dics['ArcLink'] = 'N'
            input_dics['fdsn_ic_auto'] = 'N'
            input_dics['arc_ic_auto'] = 'N'
            input_dics['fdsn_merge_auto'] = 'N'
            input_dics['arc_merge_auto'] = 'N'

    for opts in ['fdsn_update', 'arc_update']:
        if input_dics[opts] != 'N':
            input_dics['datapath'] = input_dics[opts]
            input_dics['get_events'] = 'N'
            input_dics['get_continuous'] = 'N'
            input_dics['FDSN'] = 'N'
            input_dics['ArcLink'] = 'N'

    if options.event_info:
        input_dics['FDSN'] = 'N'
        input_dics['ArcLink'] = 'N'
        input_dics['fdsn_ic_auto'] = 'N'
        input_dics['arc_ic_auto'] = 'N'
        input_dics['fdsn_merge_auto'] = 'N'
        input_dics['arc_merge_auto'] = 'N'
        input_dics['plot_all_events'] = True
        if options.identity or options.get_continuous:
            input_dics['waveform'] = 'N'
    else:
        input_dics['plot_all_events'] = False

    if options.event_info and options.get_continuous:
        input_dics['plot_all_events'] = False

    if options.seismicity:
        input_dics['FDSN'] = 'N'
        input_dics['ArcLink'] = 'N'
        input_dics['fdsn_ic_auto'] = 'N'
        input_dics['arc_ic_auto'] = 'N'
        input_dics['fdsn_merge_auto'] = 'N'
        input_dics['arc_merge_auto'] = 'N'
        input_dics['max_result'] = 1000000

    if options.FDSN == 'N':
        input_dics['fdsn_ic_auto'] = 'N'
        input_dics['fdsn_merge_auto'] = 'N'

    if options.ArcLink == 'N':
        input_dics['arc_ic_auto'] = 'N'
        input_dics['arc_merge_auto'] = 'N'

    if options.ic_no:
        input_dics['fdsn_ic_auto'] = 'N'
        input_dics['arc_ic_auto'] = 'N'

    if options.merge_no:
        input_dics['fdsn_merge_auto'] = 'N'
        input_dics['arc_merge_auto'] = 'N'

    if input_dics['plot_fdsn'] == 'Y' or input_dics['plot_arc'] == 'Y':
        input_dics['plot_all'] = 'N'

    # Create a priority list (for all requested clients)
    # For the moment, we just support: FDSN and ArcLink, but it should be
    # easily extendable!
    input_dics['priority_clients'] = []
    for cli in ['FDSN', 'ArcLink']:
        if input_dics[cli] == 'Y':
            input_dics['priority_clients'].append(cli)

    return input_dics

# ##################### input_logger ###################################


def input_logger(argus, address, inputs):
    """
    log the entered command line!
    :param argus:
    :param address:
    :param inputs:
    :return:
    """
    st_argus = 'Command line:\n-------------\n'
    for item in argus:
        st_argus += item + ' '
    st_argus += '\n\ninputs:\n-------\n'
    items = []
    for item in inputs:
        items.append(item)
    items.sort()
    for item in items:
        st_argus += '%s: %s\n' % (item, inputs[item])
    logger_open = open(address, 'w')
    logger_open.write(st_argus)
    logger_open.close()
